
/*  A Bison parser, made from c-parse.y  */

#define	IDENTIFIER	258
#define	TYPENAME	259
#define	SCSPEC	260
#define	TYPESPEC	261
#define	TYPE_QUAL	262
#define	CONSTANT	263
#define	STRING	264
#define	ELLIPSIS	265
#define	SIZEOF	266
#define	ENUM	267
#define	STRUCT	268
#define	UNION	269
#define	IF	270
#define	ELSE	271
#define	WHILE	272
#define	DO	273
#define	FOR	274
#define	SWITCH	275
#define	CASE	276
#define	DEFAULT	277
#define	BREAK	278
#define	CONTINUE	279
#define	RETURN	280
#define	GOTO	281
#define	ASM_KEYWORD	282
#define	TYPEOF	283
#define	ALIGNOF	284
#define	ALIGN	285
#define	ATTRIBUTE	286
#define	EXTENSION	287
#define	LABEL	288
#define	REALPART	289
#define	IMAGPART	290
#define	ASSIGN	291
#define	OROR	292
#define	ANDAND	293
#define	EQCOMPARE	294
#define	ARITHCOMPARE	295
#define	LSHIFT	296
#define	RSHIFT	297
#define	UNARY	298
#define	PLUSPLUS	299
#define	MINUSMINUS	300
#define	HYPERUNARY	301
#define	POINTSAT	302
#define	INTERFACE	303
#define	IMPLEMENTATION	304
#define	END	305
#define	SELECTOR	306
#define	DEFS	307
#define	ENCODE	308
#define	CLASSNAME	309
#define	PUBLIC	310
#define	PRIVATE	311
#define	PROTECTED	312
#define	PROTOCOL	313
#define	OBJECTNAME	314
#define	CLASS	315
#define	ALIAS	316
#define	OBJC_STRING	317

#line 44 "c-parse.y"

#include <stdio.h>
#include <errno.h>
#include <setjmp.h>

#include "config.h"
#include "tree.h"
#include "input.h"
#include "c-lex.h"
#include "c-tree.h"
#include "flags.h"

#ifdef MULTIBYTE_CHARS
#include <stdlib.h>
#include <locale.h>
#endif


/* Since parsers are distinct for each language, put the language string
   definition here.  */
char *language_string = "GNU C";

#ifndef errno
extern int errno;
#endif

void yyerror ();

/* Like YYERROR but do call yyerror.  */
#define YYERROR1 { yyerror ("syntax error"); YYERROR; }

/* Cause the `yydebug' variable to be defined.  */
#define YYDEBUG 1

#line 81 "c-parse.y"
typedef union {long itype; tree ttype; enum tree_code code;
	char *filename; int lineno; } YYSTYPE;
#line 192 "c-parse.y"

/* Number of statements (loosely speaking) seen so far.  */
static int stmt_count;

/* Input file and line number of the end of the body of last simple_if;
   used by the stmt-rule immediately after simple_if returns.  */
static char *if_stmt_file;
static int if_stmt_line;

/* List of types and structure classes of the current declaration.  */
static tree current_declspecs;

/* Stack of saved values of current_declspecs.  */
static tree declspec_stack;

/* 1 if we explained undeclared var errors.  */
static int undeclared_variable_notice;


/* Tell yyparse how to print a token's value, if yydebug is set.  */

#define YYPRINT(FILE,YYCHAR,YYLVAL) yyprint(FILE,YYCHAR,YYLVAL)
extern void yyprint ();

#ifndef YYLTYPE
typedef
  struct yyltype
    {
      int timestamp;
      int first_line;
      int first_column;
      int last_line;
      int last_column;
      char *text;
   }
  yyltype;

#define YYLTYPE yyltype
#endif

#include <stdio.h>

#ifndef __STDC__
#define const
#endif



#define	YYFINAL		619
#define	YYFLAG		-32768
#define	YYNTBASE	85

#define YYTRANSLATE(x) ((unsigned)(x) <= 317 ? yytranslate[x] : 221)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    81,     2,     2,     2,    53,    44,     2,    60,
    77,    51,    49,    82,    50,    59,    52,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,    39,    78,     2,
    37,     2,    38,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
    61,     2,    84,    43,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    83,    42,    79,    80,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
    36,    40,    41,    45,    46,    47,    48,    54,    55,    56,
    57,    58,    62,    63,    64,    65,    66,    67,    68,    69,
    70,    71,    72,    73,    74,    75,    76
};

static const short yyrline[] = {     0,
   218,   222,   235,   237,   237,   238,   240,   242,   243,   253,
   259,   261,   263,   265,   267,   268,   269,   274,   280,   282,
   283,   285,   290,   292,   293,   295,   300,   302,   303,   307,
   309,   312,   314,   316,   318,   320,   322,   324,   328,   332,
   335,   338,   341,   345,   347,   350,   353,   356,   360,   386,
   391,   393,   395,   397,   399,   403,   405,   408,   412,   439,
   441,   443,   445,   447,   449,   451,   453,   455,   457,   459,
   461,   463,   465,   467,   469,   471,   474,   480,   581,   582,
   584,   590,   592,   606,   629,   631,   633,   637,   643,   645,
   650,   652,   657,   659,   660,   670,   675,   677,   678,   679,
   682,   687,   691,   694,   702,   707,   709,   710,   711,   718,
   726,   731,   735,   739,   743,   745,   753,   756,   760,   762,
   764,   775,   779,   781,   784,   797,   800,   804,   806,   814,
   815,   816,   820,   822,   828,   829,   830,   833,   835,   838,
   840,   843,   846,   852,   859,   862,   868,   875,   878,   885,
   888,   893,   895,   900,   906,   907,   917,   928,   948,   950,
   955,   962,   967,   971,   974,   976,   981,   984,   986,   988,
   992,   995,   995,   998,   998,  1001,  1001,  1004,  1006,  1023,
  1027,  1044,  1051,  1053,  1058,  1061,  1066,  1068,  1070,  1072,
  1080,  1086,  1088,  1090,  1092,  1098,  1104,  1106,  1108,  1110,
  1112,  1115,  1120,  1124,  1127,  1129,  1131,  1133,  1136,  1138,
  1141,  1144,  1147,  1150,  1154,  1156,  1159,  1161,  1165,  1168,
  1173,  1175,  1177,  1191,  1197,  1202,  1207,  1212,  1216,  1218,
  1222,  1226,  1230,  1240,  1242,  1247,  1250,  1254,  1257,  1261,
  1264,  1267,  1270,  1274,  1277,  1281,  1285,  1287,  1289,  1291,
  1293,  1295,  1297,  1299,  1307,  1309,  1310,  1313,  1315,  1318,
  1321,  1332,  1334,  1339,  1341,  1344,  1358,  1361,  1364,  1366,
  1371,  1376,  1384,  1389,  1392,  1405,  1413,  1417,  1421,  1425,
  1431,  1435,  1440,  1442,  1453,  1456,  1457,  1474,  1479,  1482,
  1493,  1495,  1505,  1515,  1516,  1524,  1527,  1539,  1543,  1560,
  1567,  1576,  1578,  1583,  1588,  1592,  1596,  1607,  1614,  1621,
  1628,  1639,  1643,  1646,  1651,  1674,  1705,  1730,  1759,  1774,
  1785,  1789,  1793,  1796,  1801,  1803,  1806,  1808,  1812,  1817,
  1820,  1826,  1831,  1836,  1838,  1847,  1848,  1854,  1856,  1861,
  1863,  1867,  1870,  1876,  1879,  1881,  1883,  1885,  1892,  1897,
  1902,  1904,  1913,  1916,  1921,  1924
};

static const char * const yytname[] = {     0,
"error","$illegal.","IDENTIFIER","TYPENAME","SCSPEC","TYPESPEC","TYPE_QUAL","CONSTANT","STRING","ELLIPSIS",
"SIZEOF","ENUM","STRUCT","UNION","IF","ELSE","WHILE","DO","FOR","SWITCH",
"CASE","DEFAULT","BREAK","CONTINUE","RETURN","GOTO","ASM_KEYWORD","TYPEOF","ALIGNOF","ALIGN",
"ATTRIBUTE","EXTENSION","LABEL","REALPART","IMAGPART","ASSIGN","'='","'?'","':'","OROR",
"ANDAND","'|'","'^'","'&'","EQCOMPARE","ARITHCOMPARE","LSHIFT","RSHIFT","'+'","'-'",
"'*'","'/'","'%'","UNARY","PLUSPLUS","MINUSMINUS","HYPERUNARY","POINTSAT","'.'","'('",
"'['","INTERFACE","IMPLEMENTATION","END","SELECTOR","DEFS","ENCODE","CLASSNAME","PUBLIC","PRIVATE",
"PROTECTED","PROTOCOL","OBJECTNAME","CLASS","ALIAS","OBJC_STRING","')'","';'","'}'","'~'",
"'!'","','","'{'","']'","program"
};

static const short yyr1[] = {     0,
    85,    85,    87,    86,    88,    86,    89,    89,    89,    90,
    90,    90,    90,    90,    90,    90,    90,    92,    93,    91,
    91,    94,    95,    91,    91,    96,    97,    91,    91,    98,
    98,    99,    99,    99,    99,    99,    99,    99,   100,   101,
   101,   102,   102,   103,   103,   104,   103,   103,   103,   103,
   103,   103,   103,   103,   103,   105,   105,   106,   105,   107,
   107,   107,   107,   107,   107,   107,   107,   107,   107,   107,
   107,   107,   107,   107,   107,   107,   107,   108,   108,   108,
   108,   108,   109,   108,   108,   108,   108,   108,   108,   108,
   110,   110,   111,   111,   111,   112,   113,   113,   113,   113,
   114,   114,   114,   114,   115,   116,   116,   116,   116,   117,
   118,   118,   118,   118,   118,   118,   119,   119,   120,   120,
   120,   121,   121,   121,   121,   122,   122,   123,   123,   124,
   124,   124,   124,   124,   125,   125,   125,   126,   126,   127,
   127,   128,   128,   130,   129,   129,   132,   131,   131,   133,
   133,   134,   134,   135,   135,   135,   135,   135,   136,   137,
   136,   136,   138,   138,   139,   139,   140,   141,   140,   140,
   142,   140,   143,   140,   144,   140,   145,   140,   147,   146,
   149,   148,   150,   150,   151,   151,   151,   151,   151,   151,
   152,   152,   152,   152,   152,   153,   153,   153,   153,   153,
   153,   155,   154,   154,   154,   156,   154,   154,   154,   157,
   154,   158,   154,   154,   159,   159,   160,   160,   161,   161,
   162,   162,   162,   163,   163,   163,   163,   163,   164,   164,
   165,   165,   165,   166,   166,   167,   167,   168,   168,   169,
   169,   170,   170,   171,   171,   172,   172,   172,   172,   172,
   172,   172,   172,   172,   173,   173,   173,   174,   174,   175,
   176,   177,   177,   178,   178,   179,   180,   180,   181,   181,
   181,   181,   182,   182,   183,   185,   184,   186,   187,   188,
   188,   189,   190,   190,   191,   191,   191,   192,   191,   191,
   191,   193,   194,   191,   191,   191,   195,   196,   197,   191,
   198,   191,   191,   191,   191,   191,   191,   191,   191,   191,
   191,   191,   191,   199,   201,   200,   202,   202,   202,   202,
   203,   203,   204,   204,   205,   205,   206,   206,   207,   208,
   208,   210,   209,   211,   212,   211,   211,   213,   213,   213,
   213,   214,   214,   215,   215,   215,   215,   215,   217,   216,
   218,   218,   219,   219,   220,   220
};

static const short yyr2[] = {     0,
     0,     1,     0,     2,     0,     3,     1,     1,     5,     3,
     4,     4,     2,     2,     2,     2,     1,     0,     0,     7,
     4,     0,     0,     7,     4,     0,     0,     6,     3,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     0,
     1,     1,     3,     1,     2,     0,     3,     2,     2,     2,
     4,     2,     4,     2,     2,     1,     4,     0,     7,     1,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     5,     3,     3,     1,     1,     1,
     3,     3,     0,     4,     4,     4,     3,     3,     2,     2,
     1,     2,     0,     1,     2,     3,     1,     1,     2,     2,
     4,     4,     2,     2,     3,     1,     1,     2,     2,     0,
     4,     4,     3,     3,     2,     2,     2,     3,     0,     2,
     2,     1,     1,     2,     2,     2,     3,     0,     2,     1,
     1,     1,     4,     4,     1,     1,     1,     1,     3,     1,
     3,     0,     4,     0,     6,     3,     0,     6,     3,     0,
     6,     1,     3,     1,     1,     4,     4,     8,     1,     0,
     4,     1,     0,     2,     1,     3,     1,     0,     4,     1,
     0,     8,     0,     6,     0,     4,     0,     5,     0,     3,
     0,     3,     1,     1,     3,     3,     4,     3,     3,     1,
     3,     4,     3,     3,     1,     3,     3,     3,     4,     3,
     1,     0,     6,     4,     2,     0,     6,     4,     2,     0,
     7,     0,     6,     2,     0,     1,     0,     1,     1,     2,
     0,     3,     2,     3,     1,     3,     1,     1,     1,     3,
     4,     6,     5,     1,     3,     1,     3,     2,     2,     0,
     1,     1,     2,     0,     2,     3,     3,     2,     3,     4,
     3,     2,     3,     2,     1,     2,     2,     0,     1,     2,
     0,     0,     1,     1,     2,     3,     1,     2,     2,     6,
     5,     5,     2,     2,     4,     0,     4,     0,     0,     3,
     4,     3,     1,     1,     1,     1,     2,     0,     4,     1,
     3,     0,     0,     7,     5,     2,     0,     0,     0,    12,
     0,     6,     2,     2,     2,     3,     6,     8,    10,    12,
     3,     4,     1,     1,     0,     6,     3,     5,     2,     2,
     0,     1,     0,     1,     0,     1,     1,     3,     4,     1,
     3,     0,     2,     2,     0,     4,     2,     0,     1,     1,
     3,     1,     3,     2,     2,     2,     2,     2,     0,     2,
     1,     2,     1,     3,     1,     3
};

static const short yydefact[] = {     3,
     5,     0,     0,     0,   132,   123,   130,   122,     0,     0,
     0,     0,     0,    17,     4,     8,     7,     0,   110,   110,
   119,   131,     6,    15,    16,    30,    31,   212,   214,   221,
   205,   221,   209,     0,     0,   201,   244,     0,     0,   140,
     0,    14,     0,   125,   124,    13,     0,   119,   117,     0,
   210,     0,     0,   202,     0,   206,    78,    79,    91,     0,
     0,    46,     0,     0,     0,    32,    34,    33,     0,    35,
    36,     0,    37,    38,     0,     0,    39,    56,    60,    42,
    44,    80,   242,     0,   240,   128,     0,   240,     0,     0,
    10,     0,    29,     0,   349,     0,     0,   150,   190,   244,
     0,     0,   138,     0,   183,   184,     0,     0,   118,   121,
   135,   136,   120,   137,   236,   217,   234,     0,   204,   228,
   223,   110,   220,   110,   221,   208,   221,     0,    50,     0,
    52,     0,    54,    55,    49,    45,     0,     0,     0,     0,
    48,     0,     0,     0,     0,   323,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    89,    90,     0,     0,    40,     0,    92,   133,   244,   332,
     0,   238,   241,   126,   134,   243,   128,   239,   245,   198,
   197,   141,   142,     0,   196,     0,   200,     0,     0,    27,
     0,   278,    98,   279,     0,   149,     0,     0,    12,     0,
    21,     0,   150,   349,     0,    11,    25,     0,     0,   218,
     0,   217,   278,   222,   278,     0,     0,     0,     0,    47,
    82,    81,   261,     0,     0,     9,    43,    77,    76,   324,
     0,    74,    73,    71,    72,    70,    69,    68,    66,    67,
    61,    62,    63,    64,    65,    88,    87,     0,    41,     0,
   248,     0,   252,     0,   254,     0,   332,     0,   129,   127,
     0,     0,   353,   339,   240,   240,   351,     0,   340,   342,
   350,     0,   199,   260,     0,   100,    95,    99,     0,     0,
   147,   189,   185,   139,    19,   146,   186,   188,     0,    23,
   237,   235,   213,     0,   224,   229,   279,   226,   203,   207,
    51,    53,   269,   262,    84,    58,    57,     0,    85,    86,
   247,   246,   333,   253,   249,   251,     0,   143,   337,   195,
   244,   332,   344,   345,   346,   244,   347,   348,   334,   335,
     0,   352,     0,     0,    28,   267,    96,   110,   110,     0,
     0,     0,   144,   187,     0,   211,   278,     0,     0,     0,
   263,   264,     0,    75,   250,   248,   349,     0,   248,     0,
   341,   343,   354,   268,   103,     0,   104,     0,   154,   155,
     0,   152,   162,   160,   159,   148,    20,     0,    24,   230,
     0,   150,   355,     0,     0,     0,   278,     0,   107,   279,
   255,   265,   170,    78,     0,     0,   168,     0,   167,     0,
   215,   165,   194,   191,   193,     0,   336,     0,     0,   142,
     0,     0,     0,     0,   145,   150,     0,   231,   266,     0,
   271,   109,   108,     0,     0,   272,   257,   279,   256,     0,
     0,     0,     0,   175,    59,     0,   164,   192,   101,   102,
     0,     0,   151,   153,     0,   233,   150,   356,   270,     0,
   132,     0,   292,   276,     0,     0,     0,     0,     0,     0,
     0,     0,   321,   313,     0,     0,   105,   110,   110,   285,
   290,     0,     0,   282,   283,   286,   314,   284,   177,     0,
     0,     0,     0,   166,   156,     0,   157,   161,   232,     0,
     0,   278,   323,     0,     0,   319,   303,   304,   305,     0,
     0,     0,   322,     0,   320,   287,   115,     0,   116,     0,
     0,   274,   279,   273,   296,     0,     0,     0,   173,   169,
   176,     0,     0,     0,     0,    44,     0,     0,     0,   317,
   306,     0,   311,     0,     0,   113,   142,     0,   114,   142,
   291,   278,     0,     0,   178,     0,     0,     0,   275,     0,
   277,   315,   297,   301,     0,   312,     0,   111,     0,   112,
     0,   289,   280,   278,     0,   171,   174,     0,   293,   278,
   323,   278,   318,   325,     0,   180,   182,   281,   295,     0,
   158,   278,   316,     0,   302,     0,     0,   326,   327,   307,
   172,   294,   298,     0,   325,     0,     0,   323,     0,     0,
   308,   328,     0,   329,     0,     0,   299,   330,     0,   309,
   278,     0,     0,   300,   310,   331,     0,     0,     0
};

static const short yydefgoto[] = {   617,
     1,     2,     3,    15,    16,    17,   202,   342,   208,   345,
    97,   275,   398,    75,   230,   248,    77,    78,   132,    79,
   353,    80,    81,   139,    82,   190,   191,   192,   337,   386,
   387,    18,   467,   265,    49,   266,    85,   174,    21,   113,
   102,    39,    98,   103,   378,    40,   341,   196,   371,   372,
   376,   414,   400,   401,   402,   433,   580,   547,   483,   517,
   536,   559,   539,   561,   183,   105,   323,   106,    22,   125,
   127,   118,    50,   437,   211,    52,    53,   123,   295,   296,
   116,   117,    87,   172,    88,    89,   173,   388,   425,   193,
   304,   350,   351,   352,   335,   336,   471,   472,   473,   492,
   513,   279,   514,   391,   474,   475,   542,   491,   582,   571,
   598,   611,   572,   476,   477,   570,   478,   504,   231,   587,
   588,   589,   609,   253,   254,   267,   360,   268,   269,   270,
   185,   186,   271,   272,   384
};

static const short yypact[] = {    68,
    86,  1840,  1840,   328,-32768,-32768,-32768,-32768,    60,    70,
    77,    67,    73,-32768,-32768,-32768,-32768,    95,    42,   398,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,    53,-32768,
    61,-32768,    75,  1719,  1635,-32768,-32768,    95,    98,-32768,
   711,-32768,   294,-32768,-32768,-32768,    95,-32768,   684,   411,
-32768,    71,   261,-32768,    89,-32768,-32768,-32768,-32768,  1732,
  1782,-32768,  1719,  1719,   411,-32768,-32768,-32768,  1719,-32768,
-32768,  1033,-32768,-32768,  1719,   135,   132,-32768,-32768,  1928,
   719,   220,-32768,   150,   130,-32768,   171,  1115,   263,   360,
-32768,   294,-32768,   144,-32768,  1220,   188,   227,-32768,-32768,
   294,   205,-32768,  1213,   385,   393,   217,   873,   684,-32768,
-32768,-32768,-32768,-32768,   243,   204,-32768,   411,-32768,-32768,
-32768,   399,   216,   278,-32768,-32768,-32768,  1033,-32768,  1033,
-32768,  1719,-32768,-32768,-32768,-32768,   225,   228,   224,   233,
-32768,   237,  1719,  1719,  1719,  1719,  1719,  1719,  1719,  1719,
  1719,  1719,  1719,  1719,  1719,  1719,  1719,  1719,  1719,  1719,
-32768,-32768,   411,   411,  1719,  1719,-32768,-32768,-32768,   130,
  1274,-32768,   424,   518,-32768,-32768,-32768,-32768,-32768,   393,
-32768,-32768,   290,   316,-32768,   380,-32768,   246,   258,-32768,
    65,    41,-32768,-32768,   281,   312,   415,   373,-32768,   294,
-32768,   188,   227,-32768,  1328,-32768,-32768,   188,  1719,   411,
   272,   204,-32768,-32768,-32768,   279,   291,   295,   314,-32768,
-32768,-32768,   310,   323,  1577,-32768,  1928,  1928,  1928,-32768,
   370,  1941,  1953,  1964,  1059,  1756,  1367,   504,   442,   442,
   421,   421,-32768,-32768,-32768,-32768,-32768,   340,   132,   348,
   218,   375,-32768,   536,-32768,   358,-32768,  1341,-32768,   518,
    40,   371,-32768,-32768,   158,   954,-32768,   374,   226,-32768,
-32768,    97,-32768,-32768,    25,-32768,-32768,-32768,  1307,   364,
-32768,   385,-32768,-32768,-32768,   422,-32768,-32768,   377,-32768,
  1928,-32768,-32768,   379,   387,-32768,-32768,   387,-32768,-32768,
-32768,-32768,-32768,   447,-32768,-32768,-32768,  1719,-32768,-32768,
   424,-32768,-32768,-32768,-32768,-32768,   404,-32768,-32768,-32768,
-32768,   127,   438,   393,-32768,-32768,   393,-32768,-32768,-32768,
  1253,-32768,   493,   224,-32768,-32768,-32768,   425,   625,   313,
   478,    25,-32768,-32768,    25,-32768,-32768,   166,   411,   651,
   447,-32768,   318,  1551,-32768,   252,-32768,  1395,   160,   536,
-32768,-32768,-32768,-32768,-32768,   294,-32768,    95,   448,-32768,
   146,-32768,-32768,-32768,  1928,-32768,-32768,   478,-32768,-32768,
  1719,    22,-32768,   253,   427,   568,   423,   732,-32768,-32768,
-32768,-32768,-32768,   472,   411,  1719,-32768,   477,  1928,   439,
   435,-32768,   438,-32768,-32768,   437,-32768,   266,   293,   122,
   151,   449,   313,   318,-32768,  1885,  1719,-32768,-32768,   411,
-32768,-32768,-32768,   813,   456,-32768,-32768,-32768,-32768,  1477,
   502,  1861,   318,-32768,-32768,  1091,-32768,-32768,-32768,-32768,
   195,   468,-32768,-32768,   483,-32768,  1885,-32768,-32768,  1527,
   508,   503,-32768,-32768,   505,   506,  1719,   529,   500,   520,
  1669,    56,   577,-32768,   560,   523,-32768,   527,   792,-32768,
   590,   894,    32,-32768,-32768,-32768,-32768,-32768,-32768,  1719,
   570,   531,  1152,-32768,-32768,   600,-32768,-32768,-32768,  1719,
   551,-32768,  1719,  1719,  1421,-32768,-32768,-32768,-32768,   537,
  1719,   542,-32768,   554,-32768,-32768,-32768,   294,-32768,    95,
   975,-32768,-32768,-32768,-32768,  1719,  1152,  1836,-32768,-32768,
-32768,   539,   545,  1719,   608,   305,   548,   550,  1719,-32768,
-32768,   555,-32768,  1719,   298,-32768,     2,   347,-32768,   125,
-32768,-32768,  1527,   558,-32768,   599,  1152,   632,-32768,   564,
-32768,-32768,-32768,-32768,  1910,-32768,     7,-32768,   224,-32768,
   224,-32768,-32768,-32768,   565,-32768,-32768,   567,-32768,-32768,
  1719,-32768,-32768,   633,   572,-32768,-32768,-32768,-32768,  1152,
-32768,-32768,-32768,   583,-32768,   585,    18,   602,-32768,-32768,
-32768,-32768,-32768,  1719,   633,   603,   633,  1719,   605,    26,
-32768,-32768,   610,-32768,   316,   615,-32768,   220,   199,-32768,
-32768,   616,   316,-32768,-32768,   220,   688,   699,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,-32768,   701,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,    -7,-32768,   -34,-32768,   540,   454,-32768,   -19,
-32768,    87,   215,-32768,  -175,  -102,   517,-32768,-32768,   326,
-32768,    -6,-32768,     9,   662,    14,   661,   543,    -5,  -122,
  -342,   -40,   -96,   -61,-32768,-32768,-32768,  -183,-32768,   306,
   343,-32768,  -324,-32768,  -382,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,   -37,   -66,   366,   -13,   -10,-32768,
-32768,-32768,-32768,-32768,   514,    15,-32768,-32768,   512,   381,
   612,   534,    17,   -69,   692,   -72,  -149,   350,-32768,  -173,
-32768,-32768,-32768,   395,  -305,  -112,-32768,-32768,-32768,-32768,
   -87,  -256,  -460,  -333,-32768,   219,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,-32768,   222,-32768,  -457,   165,
-32768,   172,-32768,   511,-32768,  -231,-32768,-32768,-32768,   432,
  -182,-32768,-32768,-32768,-32768
};


#define	YYLAST		2017


static const short yytable[] = {    76,
    84,    29,    31,    33,    41,   104,   107,   203,   261,   194,
    19,    19,    43,    47,    48,    20,    20,   276,   178,   286,
   252,   287,   313,   408,    90,   334,   224,   197,    94,    86,
   182,   525,   515,   108,   198,   527,   377,   138,   114,   379,
   348,   -94,   115,   133,   134,   574,    55,    86,   167,   136,
   277,   259,   195,   484,   429,   141,   595,   135,    26,    27,
   417,   188,    26,    27,   605,   189,    86,    -1,   -97,   -97,
   -97,   -97,    26,    27,   -97,   180,   -97,   -97,   -97,    26,
    27,   562,   177,   575,  -179,    -2,   203,    90,   140,   445,
   429,   516,   -97,   138,   596,   138,   251,    36,   114,   285,
   521,   311,   606,   578,   194,   290,   501,   223,   482,   583,
   115,   585,   220,   584,   194,   213,   318,   215,   177,    42,
   194,   592,    86,   -94,    86,   297,    34,   297,   407,    36,
   282,   250,    35,   430,   545,    51,   256,   259,   284,   216,
   603,   217,    28,    54,   218,    37,   219,   -97,    94,   119,
   614,    94,    30,   441,    38,   246,   247,    56,   442,    32,
    36,   320,    36,   114,   567,   535,   179,   126,    36,    99,
   289,   450,   252,   332,   404,    91,   389,   326,   333,    92,
   169,    95,    96,   180,    95,    96,   322,   171,   189,   170,
   171,  -278,  -278,  -278,  -278,   325,   328,   591,   418,  -278,
  -278,  -278,   115,   184,   381,   307,   311,  -181,   321,   311,
   326,   142,   422,   143,   427,  -278,   100,   322,   171,   322,
   171,   364,   412,   317,   179,   101,   168,   413,   167,   227,
   228,   229,   446,   232,   233,   234,   235,   236,   237,   238,
   239,   240,   241,   242,   243,   244,   245,   175,   356,   114,
   427,   324,   327,   359,    36,   320,   543,   195,   179,   297,
    48,   120,   390,   489,     5,    36,     7,    83,   169,   179,
   -93,   485,     9,    10,    11,   612,   486,   170,   171,   209,
   613,     5,   199,     7,   176,   210,   200,   338,    13,     9,
    10,    11,   339,   214,   206,   291,    36,    99,    92,   390,
   428,   221,   321,   330,   222,    13,   223,   331,    90,   225,
   382,   322,   171,    37,   226,   369,    94,   470,   393,   370,
   394,    27,    38,   406,    59,    58,    59,   409,    60,   273,
   419,   366,   368,    48,   420,   274,   428,   470,   121,  -219,
   280,   383,   180,   439,   100,   180,    61,   200,   281,    62,
   293,    63,    64,   101,   410,  -227,  -227,   299,    65,   161,
   162,    66,   163,   164,   165,   166,    67,    68,    69,   300,
   440,   301,    70,    71,    92,   558,   395,    72,   396,   200,
   262,   552,   263,     5,     6,     7,     8,   431,   303,   264,
   302,     9,    10,    11,   354,   466,  -163,    73,    74,   305,
   397,     5,    44,     7,    45,    24,    25,    13,   308,     9,
    10,    11,   448,    26,    27,   466,   309,    36,    99,    95,
    96,   179,   465,   340,   560,    13,   500,   375,    92,   608,
   470,   310,   204,   205,   257,   258,   181,   616,   468,   399,
   203,   314,   465,   469,   204,   205,   576,   319,   577,   283,
   329,   312,    95,    96,   502,   523,  -338,   346,   343,   528,
   344,   508,   510,    48,   375,   100,   532,   416,   347,   538,
   537,   158,   159,   160,   101,    46,  -225,  -225,   373,   349,
    57,   544,   432,   257,   258,    58,    59,   355,    60,   550,
   156,   157,   158,   159,   160,   363,   540,   357,   358,   557,
   399,  -258,   365,   447,   274,   421,    61,   411,   466,    62,
   -30,    63,    64,   129,   131,   434,   436,   435,    65,   399,
   438,    66,   399,   111,   112,   443,    67,    68,    69,     9,
    10,    11,    70,    71,   449,   465,   262,    72,   479,     5,
     6,     7,     8,   495,   487,   264,   -31,     9,    10,    11,
   154,   155,   156,   157,   158,   159,   160,    73,    74,   599,
   374,   488,   490,    13,   493,   494,   518,   496,   189,   399,
  -106,  -106,  -106,  -106,  -106,  -106,  -106,   497,  -106,  -106,
  -106,  -106,  -106,   503,  -106,  -106,  -106,  -106,  -106,  -106,
  -106,  -106,  -106,  -106,  -106,  -106,  -106,   498,   505,  -106,
   506,  -106,  -106,   399,   507,   511,   519,   522,  -106,   520,
   524,  -106,  -338,   534,   531,   555,  -106,  -106,  -106,   533,
   548,   549,  -106,  -106,   551,   553,   554,  -106,     5,    44,
     7,    45,   556,   399,   565,   566,     9,    10,    11,   568,
   569,   586,   579,   581,   594,  -106,  -106,  -106,  -106,   590,
  -106,   385,    13,  -278,  -278,  -278,  -278,  -278,  -278,  -278,
   593,  -278,  -278,  -278,  -278,  -278,   399,  -278,  -278,  -278,
  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,
   601,   604,  -278,   597,  -278,  -278,   607,   618,   110,   111,
   112,  -278,   610,   615,  -278,     9,    10,    11,   619,  -278,
  -278,  -278,   367,    23,   249,  -278,  -278,   526,   278,   109,
  -278,    93,   423,   122,   -26,   -26,   -26,   -26,   444,   260,
   415,   403,   -26,   -26,   -26,   294,   298,   380,  -278,   212,
  -278,  -278,   189,  -278,  -278,  -278,   424,    94,   -26,  -278,
  -278,  -142,  -278,   292,   124,   392,  -278,  -142,  -278,  -278,
  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,   600,
  -278,   563,   362,  -278,   564,  -278,  -278,   315,   602,     0,
    95,    96,  -278,   161,   162,  -278,   163,   164,   165,   166,
  -278,  -278,  -278,     0,     0,     0,  -278,  -278,  -142,     0,
     0,  -278,  -142,   -26,     0,     5,    44,     7,    45,     0,
     0,     0,     0,     9,    10,    11,     0,     0,     0,  -278,
   426,  -278,  -278,   189,  -278,  -278,  -278,     0,     0,    13,
  -278,  -278,     0,  -278,     0,     0,     0,  -278,     0,  -278,
  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,
     0,  -278,     0,     0,  -278,     0,  -278,  -278,     0,     0,
     0,     0,     0,  -278,     0,     0,  -278,     0,     0,     0,
     0,  -278,  -278,  -278,     0,     0,     0,  -278,  -278,   509,
     0,     0,  -278,   207,     0,     0,   -22,   -22,   -22,   -22,
     0,     0,     0,     0,   -22,   -22,   -22,     0,     0,     0,
  -278,  -259,  -278,  -278,   512,  -278,  -278,  -278,     0,    94,
   -22,  -278,  -278,  -142,  -278,     0,     0,     0,  -278,  -142,
  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,  -278,
  -278,     0,  -278,     0,     0,  -278,     0,  -278,  -278,     0,
     0,     0,    95,    96,  -278,     0,     0,  -278,     0,     0,
     0,     0,  -278,  -278,  -278,     0,     0,     0,  -278,  -278,
  -142,     0,     0,  -278,  -142,   -22,    36,     5,    44,     7,
    45,     0,     0,     0,     0,     9,    10,    11,     0,     0,
     0,  -278,     0,  -278,  -278,   541,  -278,  -288,  -288,     0,
     0,    13,  -288,  -288,     0,  -288,     0,     0,     0,  -288,
     0,  -288,  -288,  -288,  -288,  -288,  -288,  -288,  -288,  -288,
  -288,  -288,     0,  -288,   326,     0,  -288,     0,  -288,  -288,
     0,     0,     0,   322,   171,  -288,     0,     0,  -288,     0,
     0,     0,     0,  -288,  -288,  -288,     0,     0,     0,  -288,
  -288,     0,     0,   137,  -288,    57,     5,     0,     7,    83,
    58,    59,     0,    60,     9,    10,    11,     0,     0,     0,
     0,     0,  -288,     0,  -288,  -288,     0,  -288,     0,     0,
    13,    61,     0,     0,    62,     0,    63,    64,     0,     0,
     0,     0,     0,    65,     0,     0,    66,     0,     0,     0,
     0,    67,    68,    69,     0,     0,     0,    70,    71,     0,
     0,   393,    72,   394,    27,     0,     0,     0,    58,    59,
     0,    60,   151,   152,   153,   154,   155,   156,   157,   158,
   159,   160,    73,    74,     0,   -83,     0,     0,     5,    61,
     7,   176,    62,     0,    63,    64,     9,    10,    11,     0,
     0,    65,     0,     0,    66,     0,     0,     0,     0,    67,
    68,    69,    13,     0,     0,    70,    71,     0,     0,   395,
    72,   396,   393,     0,   394,    27,     0,     0,     0,    58,
    59,     0,    60,     0,     0,   169,     0,     0,     0,  -216,
    73,    74,     0,   397,   170,   171,     0,     0,     0,     0,
    61,     0,     0,    62,     0,    63,    64,     0,     0,     0,
     0,     0,    65,     0,     0,    66,     0,     0,     0,     0,
    67,    68,    69,     0,     0,     0,    70,    71,     0,     0,
   395,    72,   396,   201,     0,     0,   -18,   -18,   -18,   -18,
     0,     0,    57,     0,   -18,   -18,   -18,    58,    59,     0,
    60,    73,    74,     0,   397,     0,     0,     0,     0,    94,
   -18,     0,     0,  -142,     0,     0,     0,     0,    61,  -142,
     0,    62,     0,    63,    64,     0,     5,     6,     7,     8,
    65,     0,   361,    66,     9,    10,    11,     0,    67,    68,
    69,     0,     0,     0,    70,    71,    57,     0,     0,    72,
    13,    58,    59,     0,    60,     0,     0,     0,     0,     0,
  -142,     0,     0,     0,  -142,   -18,     0,     0,     0,    73,
    74,     0,    61,   187,     0,    62,     0,    63,    64,     0,
     5,     6,     7,     8,    65,     0,     0,    66,     9,    10,
    11,     0,    67,    68,    69,     0,     0,     0,    70,    71,
    57,     0,     0,    72,    13,    58,    59,     0,    60,     0,
     0,     0,     0,    57,     0,     0,     0,     0,    58,    59,
     0,    60,     0,    73,    74,     0,    61,   255,     0,    62,
     0,    63,    64,     0,     0,     0,     0,     0,    65,    61,
     0,    66,    62,     0,    63,    64,    67,    68,    69,     0,
     0,    65,    70,    71,    66,     0,     0,    72,     0,    67,
    68,    69,     0,     0,     0,    70,    71,    57,     0,     0,
    72,     0,    58,    59,     0,    60,     0,    73,    74,     0,
     0,   288,   153,   154,   155,   156,   157,   158,   159,   160,
    73,    74,     0,    61,   316,     0,    62,     0,    63,    64,
   529,     0,     0,     0,     0,    65,     0,     0,    66,     0,
     0,     0,     0,    67,    68,    69,     0,     0,     0,    70,
    71,     0,     0,     0,    72,     0,   144,   145,   146,   530,
   147,   148,   149,   150,   151,   152,   153,   154,   155,   156,
   157,   158,   159,   160,    73,    74,     0,     0,   405,   394,
   451,     6,     7,     8,    58,    59,     0,    60,     9,    10,
    11,   452,     0,   453,   454,   455,   456,   457,   458,   459,
   460,   461,   462,   463,    13,    61,     0,     0,    62,     0,
    63,    64,     0,     0,     0,     0,     0,    65,     0,     0,
    66,     0,     0,     0,     0,    67,    68,    69,     0,   394,
    27,    70,    71,     0,    58,    59,    72,    60,     0,     0,
     0,   452,     0,   453,   454,   455,   456,   457,   458,   459,
   460,   461,   462,   463,   464,    61,    73,    74,    62,   223,
    63,    64,     0,     0,     0,     0,     0,    65,     0,     0,
    66,     0,     0,     0,     0,    67,    68,    69,     0,    57,
     0,    70,    71,     0,    58,    59,    72,    60,   146,     0,
   147,   148,   149,   150,   151,   152,   153,   154,   155,   156,
   157,   158,   159,   160,   464,    61,    73,    74,    62,   223,
    63,    64,     0,     0,     0,     0,     0,    65,     0,     0,
    66,     0,     0,     0,     0,    67,    68,    69,     0,     0,
     0,    70,    71,     0,     0,     0,    72,    57,     5,     0,
     7,    83,    58,    59,     0,    60,     9,    10,    11,     0,
     0,     0,     0,     0,     0,     0,    73,    74,     0,   306,
     0,     0,    13,    61,     0,     0,    62,     0,    63,    64,
     0,    57,     0,     0,     0,    65,    58,    59,    66,    60,
     0,     0,     0,    67,    68,    69,     0,     0,     0,    70,
    71,     0,     0,     0,    72,     0,     0,    61,     0,     0,
    62,     0,    63,    64,     0,     0,     0,     0,     0,    65,
     0,     0,    66,     0,    73,    74,     0,    67,    68,    69,
     0,    57,     0,    70,    71,     0,    58,    59,    72,    60,
     0,     0,     0,     0,    57,     0,     0,     0,     0,    58,
    59,     0,    60,     0,     0,     0,   499,    61,    73,    74,
    62,     0,    63,    64,     0,     0,     0,     0,     0,    65,
    61,     0,    66,    62,     0,    63,    64,    67,    68,    69,
     0,     0,    65,    70,    71,    66,     0,     0,    72,     0,
    67,    68,    69,     0,    57,     0,    70,    71,     0,    58,
    59,   128,    60,     0,     0,     0,     0,     0,    73,    74,
   152,   153,   154,   155,   156,   157,   158,   159,   160,     0,
    61,    73,    74,    62,     0,    63,    64,     0,     0,     0,
     0,     0,    65,     0,     0,    66,     0,     0,     0,     0,
    67,    68,    69,     0,     0,     0,    70,    71,     0,     0,
     4,   130,  -110,     5,     6,     7,     8,     0,     0,     0,
     0,     9,    10,    11,     0,     0,     0,     0,     0,     0,
     0,    73,    74,     0,     0,     0,    12,    13,     0,     0,
   480,   144,   145,   146,     0,   147,   148,   149,   150,   151,
   152,   153,   154,   155,   156,   157,   158,   159,   160,     0,
  -110,     0,     0,     0,     0,     0,   144,   145,   146,  -110,
   147,   148,   149,   150,   151,   152,   153,   154,   155,   156,
   157,   158,   159,   160,     0,   195,     0,    14,     0,   546,
   144,   145,   146,     0,   147,   148,   149,   150,   151,   152,
   153,   154,   155,   156,   157,   158,   159,   160,     0,     0,
     0,     0,     0,     0,   481,   144,   145,   146,   573,   147,
   148,   149,   150,   151,   152,   153,   154,   155,   156,   157,
   158,   159,   160,   144,   145,   146,     0,   147,   148,   149,
   150,   151,   152,   153,   154,   155,   156,   157,   158,   159,
   160,   148,   149,   150,   151,   152,   153,   154,   155,   156,
   157,   158,   159,   160,   149,   150,   151,   152,   153,   154,
   155,   156,   157,   158,   159,   160,   150,   151,   152,   153,
   154,   155,   156,   157,   158,   159,   160
};

static const short yycheck[] = {    34,
    35,     9,    10,    11,    18,    43,    47,   104,   184,    97,
     2,     3,    19,    20,    20,     2,     3,   191,    88,   203,
   170,   204,   254,   366,    38,     1,   139,   100,    27,    35,
    92,   492,     1,    47,   101,   493,   342,    72,    49,   345,
   297,     1,    50,    63,    64,    39,    32,    53,     9,    69,
    10,   174,    31,   436,   388,    75,    39,    65,     3,     4,
    39,    96,     3,     4,    39,     1,    72,     0,     4,     5,
     6,     7,     3,     4,    10,    89,    12,    13,    14,     3,
     4,   542,    88,    77,    83,     0,   183,   101,    72,   414,
   424,    60,    28,   128,    77,   130,   169,     3,   109,   202,
   483,   251,    77,   564,   192,   208,    51,    83,   433,   570,
   118,   572,   132,   571,   202,   122,    77,   124,   124,    78,
   208,   582,   128,    83,   130,   213,    60,   215,   360,     3,
   197,   166,    60,   390,   517,    83,   171,   260,   200,   125,
   598,   127,    83,    83,   128,    51,   130,    83,    27,    79,
   611,    27,    83,     3,    60,   163,   164,    83,     8,    83,
     3,     4,     3,   174,   547,   508,     7,    79,     3,     4,
   205,   428,   322,    77,   357,    78,   350,    51,    82,    82,
    51,    60,    61,   197,    60,    61,    60,    61,     1,    60,
    61,     4,     5,     6,     7,   265,   266,   580,   382,    12,
    13,    14,   210,    60,    39,   225,   356,    83,    51,   359,
    51,    77,   386,    82,   388,    28,    51,    60,    61,    60,
    61,   334,    77,   258,     7,    60,    77,    82,     9,   143,
   144,   145,   416,   147,   148,   149,   150,   151,   152,   153,
   154,   155,   156,   157,   158,   159,   160,    77,   321,   260,
   424,   265,   266,   326,     3,     4,   513,    31,     7,   347,
   266,     1,   350,   447,     4,     3,     6,     7,    51,     7,
    83,    77,    12,    13,    14,    77,    82,    60,    61,    37,
    82,     4,    78,     6,     7,    82,    82,   279,    28,    12,
    13,    14,   279,    78,    78,   209,     3,     4,    82,   387,
   388,    77,    51,    78,    77,    28,    83,    82,   322,    77,
   348,    60,    61,    51,    78,     3,    27,   430,     1,     7,
     3,     4,    60,   358,     9,     8,     9,   368,    11,    84,
    78,   338,   339,   339,    82,    78,   424,   450,    78,    79,
    60,   349,   356,    78,    51,   359,    29,    82,    37,    32,
    79,    34,    35,    60,   368,    78,    79,    79,    41,    55,
    56,    44,    58,    59,    60,    61,    49,    50,    51,    79,
    78,    77,    55,    56,    82,    78,    59,    60,    61,    82,
     1,    77,     3,     4,     5,     6,     7,   395,    79,    10,
    77,    12,    13,    14,   308,   430,    79,    80,    81,    77,
    83,     4,     5,     6,     7,    78,    79,    28,    39,    12,
    13,    14,   420,     3,     4,   450,    77,     3,     4,    60,
    61,     7,   430,    60,    78,    28,   461,   341,    82,   605,
   543,    84,    60,    61,    60,    61,    77,   613,   430,   353,
   537,    84,   450,   430,    60,    61,   559,    77,   561,    77,
    77,    77,    60,    61,   462,   490,    77,    79,    37,   494,
    84,   468,   469,   469,   378,    51,   501,   381,    82,   510,
   508,    51,    52,    53,    60,    78,    78,    79,     1,    33,
     3,   516,   396,    60,    61,     8,     9,    84,    11,   524,
    49,    50,    51,    52,    53,     3,   510,    60,    61,   534,
   414,    79,    78,   417,    78,    79,    29,    60,   543,    32,
    39,    34,    35,    60,    61,    39,    82,    79,    41,   433,
    84,    44,   436,     6,     7,    77,    49,    50,    51,    12,
    13,    14,    55,    56,    79,   543,     1,    60,    37,     4,
     5,     6,     7,   457,    77,    10,    39,    12,    13,    14,
    47,    48,    49,    50,    51,    52,    53,    80,    81,   594,
    83,    79,    60,    28,    60,    60,   480,    39,     1,   483,
     3,     4,     5,     6,     7,     8,     9,    78,    11,    12,
    13,    14,    15,     7,    17,    18,    19,    20,    21,    22,
    23,    24,    25,    26,    27,    28,    29,    78,    39,    32,
    78,    34,    35,   517,    78,    16,    37,     8,    41,    79,
    60,    44,    77,    60,    78,   529,    49,    50,    51,    78,
    82,    77,    55,    56,    17,    78,    77,    60,     4,     5,
     6,     7,    78,   547,    77,    37,    12,    13,    14,     8,
    77,     9,    78,    77,    60,    78,    79,    80,    81,    78,
    83,     1,    28,     3,     4,     5,     6,     7,     8,     9,
    78,    11,    12,    13,    14,    15,   580,    17,    18,    19,
    20,    21,    22,    23,    24,    25,    26,    27,    28,    29,
    78,    77,    32,    82,    34,    35,    77,     0,     5,     6,
     7,    41,    78,    78,    44,    12,    13,    14,     0,    49,
    50,    51,    78,     3,   165,    55,    56,   493,   192,    48,
    60,     1,   387,    53,     4,     5,     6,     7,   413,   177,
   378,   356,    12,    13,    14,   212,   215,   347,    78,   118,
    80,    81,     1,    83,     3,     4,   387,    27,    28,     8,
     9,    31,    11,   210,    53,   351,    15,    37,    17,    18,
    19,    20,    21,    22,    23,    24,    25,    26,    27,   595,
    29,   543,   331,    32,   543,    34,    35,   257,   597,    -1,
    60,    61,    41,    55,    56,    44,    58,    59,    60,    61,
    49,    50,    51,    -1,    -1,    -1,    55,    56,    78,    -1,
    -1,    60,    82,    83,    -1,     4,     5,     6,     7,    -1,
    -1,    -1,    -1,    12,    13,    14,    -1,    -1,    -1,    78,
    79,    80,    81,     1,    83,     3,     4,    -1,    -1,    28,
     8,     9,    -1,    11,    -1,    -1,    -1,    15,    -1,    17,
    18,    19,    20,    21,    22,    23,    24,    25,    26,    27,
    -1,    29,    -1,    -1,    32,    -1,    34,    35,    -1,    -1,
    -1,    -1,    -1,    41,    -1,    -1,    44,    -1,    -1,    -1,
    -1,    49,    50,    51,    -1,    -1,    -1,    55,    56,    78,
    -1,    -1,    60,     1,    -1,    -1,     4,     5,     6,     7,
    -1,    -1,    -1,    -1,    12,    13,    14,    -1,    -1,    -1,
    78,    79,    80,    81,     1,    83,     3,     4,    -1,    27,
    28,     8,     9,    31,    11,    -1,    -1,    -1,    15,    37,
    17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
    27,    -1,    29,    -1,    -1,    32,    -1,    34,    35,    -1,
    -1,    -1,    60,    61,    41,    -1,    -1,    44,    -1,    -1,
    -1,    -1,    49,    50,    51,    -1,    -1,    -1,    55,    56,
    78,    -1,    -1,    60,    82,    83,     3,     4,     5,     6,
     7,    -1,    -1,    -1,    -1,    12,    13,    14,    -1,    -1,
    -1,    78,    -1,    80,    81,     1,    83,     3,     4,    -1,
    -1,    28,     8,     9,    -1,    11,    -1,    -1,    -1,    15,
    -1,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    -1,    29,    51,    -1,    32,    -1,    34,    35,
    -1,    -1,    -1,    60,    61,    41,    -1,    -1,    44,    -1,
    -1,    -1,    -1,    49,    50,    51,    -1,    -1,    -1,    55,
    56,    -1,    -1,     1,    60,     3,     4,    -1,     6,     7,
     8,     9,    -1,    11,    12,    13,    14,    -1,    -1,    -1,
    -1,    -1,    78,    -1,    80,    81,    -1,    83,    -1,    -1,
    28,    29,    -1,    -1,    32,    -1,    34,    35,    -1,    -1,
    -1,    -1,    -1,    41,    -1,    -1,    44,    -1,    -1,    -1,
    -1,    49,    50,    51,    -1,    -1,    -1,    55,    56,    -1,
    -1,     1,    60,     3,     4,    -1,    -1,    -1,     8,     9,
    -1,    11,    44,    45,    46,    47,    48,    49,    50,    51,
    52,    53,    80,    81,    -1,    83,    -1,    -1,     4,    29,
     6,     7,    32,    -1,    34,    35,    12,    13,    14,    -1,
    -1,    41,    -1,    -1,    44,    -1,    -1,    -1,    -1,    49,
    50,    51,    28,    -1,    -1,    55,    56,    -1,    -1,    59,
    60,    61,     1,    -1,     3,     4,    -1,    -1,    -1,     8,
     9,    -1,    11,    -1,    -1,    51,    -1,    -1,    -1,    79,
    80,    81,    -1,    83,    60,    61,    -1,    -1,    -1,    -1,
    29,    -1,    -1,    32,    -1,    34,    35,    -1,    -1,    -1,
    -1,    -1,    41,    -1,    -1,    44,    -1,    -1,    -1,    -1,
    49,    50,    51,    -1,    -1,    -1,    55,    56,    -1,    -1,
    59,    60,    61,     1,    -1,    -1,     4,     5,     6,     7,
    -1,    -1,     3,    -1,    12,    13,    14,     8,     9,    -1,
    11,    80,    81,    -1,    83,    -1,    -1,    -1,    -1,    27,
    28,    -1,    -1,    31,    -1,    -1,    -1,    -1,    29,    37,
    -1,    32,    -1,    34,    35,    -1,     4,     5,     6,     7,
    41,    -1,    10,    44,    12,    13,    14,    -1,    49,    50,
    51,    -1,    -1,    -1,    55,    56,     3,    -1,    -1,    60,
    28,     8,     9,    -1,    11,    -1,    -1,    -1,    -1,    -1,
    78,    -1,    -1,    -1,    82,    83,    -1,    -1,    -1,    80,
    81,    -1,    29,    84,    -1,    32,    -1,    34,    35,    -1,
     4,     5,     6,     7,    41,    -1,    -1,    44,    12,    13,
    14,    -1,    49,    50,    51,    -1,    -1,    -1,    55,    56,
     3,    -1,    -1,    60,    28,     8,     9,    -1,    11,    -1,
    -1,    -1,    -1,     3,    -1,    -1,    -1,    -1,     8,     9,
    -1,    11,    -1,    80,    81,    -1,    29,    84,    -1,    32,
    -1,    34,    35,    -1,    -1,    -1,    -1,    -1,    41,    29,
    -1,    44,    32,    -1,    34,    35,    49,    50,    51,    -1,
    -1,    41,    55,    56,    44,    -1,    -1,    60,    -1,    49,
    50,    51,    -1,    -1,    -1,    55,    56,     3,    -1,    -1,
    60,    -1,     8,     9,    -1,    11,    -1,    80,    81,    -1,
    -1,    84,    46,    47,    48,    49,    50,    51,    52,    53,
    80,    81,    -1,    29,    84,    -1,    32,    -1,    34,    35,
    10,    -1,    -1,    -1,    -1,    41,    -1,    -1,    44,    -1,
    -1,    -1,    -1,    49,    50,    51,    -1,    -1,    -1,    55,
    56,    -1,    -1,    -1,    60,    -1,    36,    37,    38,    39,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
    50,    51,    52,    53,    80,    81,    -1,    -1,    84,     3,
     4,     5,     6,     7,     8,     9,    -1,    11,    12,    13,
    14,    15,    -1,    17,    18,    19,    20,    21,    22,    23,
    24,    25,    26,    27,    28,    29,    -1,    -1,    32,    -1,
    34,    35,    -1,    -1,    -1,    -1,    -1,    41,    -1,    -1,
    44,    -1,    -1,    -1,    -1,    49,    50,    51,    -1,     3,
     4,    55,    56,    -1,     8,     9,    60,    11,    -1,    -1,
    -1,    15,    -1,    17,    18,    19,    20,    21,    22,    23,
    24,    25,    26,    27,    78,    29,    80,    81,    32,    83,
    34,    35,    -1,    -1,    -1,    -1,    -1,    41,    -1,    -1,
    44,    -1,    -1,    -1,    -1,    49,    50,    51,    -1,     3,
    -1,    55,    56,    -1,     8,     9,    60,    11,    38,    -1,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
    50,    51,    52,    53,    78,    29,    80,    81,    32,    83,
    34,    35,    -1,    -1,    -1,    -1,    -1,    41,    -1,    -1,
    44,    -1,    -1,    -1,    -1,    49,    50,    51,    -1,    -1,
    -1,    55,    56,    -1,    -1,    -1,    60,     3,     4,    -1,
     6,     7,     8,     9,    -1,    11,    12,    13,    14,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    80,    81,    -1,    83,
    -1,    -1,    28,    29,    -1,    -1,    32,    -1,    34,    35,
    -1,     3,    -1,    -1,    -1,    41,     8,     9,    44,    11,
    -1,    -1,    -1,    49,    50,    51,    -1,    -1,    -1,    55,
    56,    -1,    -1,    -1,    60,    -1,    -1,    29,    -1,    -1,
    32,    -1,    34,    35,    -1,    -1,    -1,    -1,    -1,    41,
    -1,    -1,    44,    -1,    80,    81,    -1,    49,    50,    51,
    -1,     3,    -1,    55,    56,    -1,     8,     9,    60,    11,
    -1,    -1,    -1,    -1,     3,    -1,    -1,    -1,    -1,     8,
     9,    -1,    11,    -1,    -1,    -1,    78,    29,    80,    81,
    32,    -1,    34,    35,    -1,    -1,    -1,    -1,    -1,    41,
    29,    -1,    44,    32,    -1,    34,    35,    49,    50,    51,
    -1,    -1,    41,    55,    56,    44,    -1,    -1,    60,    -1,
    49,    50,    51,    -1,     3,    -1,    55,    56,    -1,     8,
     9,    60,    11,    -1,    -1,    -1,    -1,    -1,    80,    81,
    45,    46,    47,    48,    49,    50,    51,    52,    53,    -1,
    29,    80,    81,    32,    -1,    34,    35,    -1,    -1,    -1,
    -1,    -1,    41,    -1,    -1,    44,    -1,    -1,    -1,    -1,
    49,    50,    51,    -1,    -1,    -1,    55,    56,    -1,    -1,
     1,    60,     3,     4,     5,     6,     7,    -1,    -1,    -1,
    -1,    12,    13,    14,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    80,    81,    -1,    -1,    -1,    27,    28,    -1,    -1,
    10,    36,    37,    38,    -1,    40,    41,    42,    43,    44,
    45,    46,    47,    48,    49,    50,    51,    52,    53,    -1,
    51,    -1,    -1,    -1,    -1,    -1,    36,    37,    38,    60,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
    50,    51,    52,    53,    -1,    31,    -1,    78,    -1,    84,
    36,    37,    38,    -1,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    50,    51,    52,    53,    -1,    -1,
    -1,    -1,    -1,    -1,    84,    36,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,    50,
    51,    52,    53,    36,    37,    38,    -1,    40,    41,    42,
    43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
    53,    41,    42,    43,    44,    45,    46,    47,    48,    49,
    50,    51,    52,    53,    42,    43,    44,    45,    46,    47,
    48,    49,    50,    51,    52,    53,    43,    44,    45,    46,
    47,    48,    49,    50,    51,    52,    53
};
#define YYPURE 1

/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "bison.simple"

/* Skeleton output parser for bison,
   Copyright (C) 1984 Bob Corbett and Richard Stallman

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__)
#include <alloca.h>
#endif

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYFAIL		goto yyerrlab;
#define YYACCEPT	return(0)
#define YYABORT 	return(1)
#define YYERROR		goto yyerrlab

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYIMPURE
#define YYLEX		yylex()
#endif

#ifndef YYPURE
#define YYLEX		yylex(&yylval, &yylloc)
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYIMPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/

int yynerrs;			/*  number of parse errors so far       */
#endif  /* YYIMPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYMAXDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYMAXDEPTH
#define YYMAXDEPTH 200
#endif

/*  YYMAXLIMIT is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#ifndef YYMAXLIMIT
#define YYMAXLIMIT 10000
#endif


#line 90 "bison.simple"
int
yyparse()
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  YYLTYPE *yylsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYMAXDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYMAXDEPTH];	/*  the semantic value stack		*/
  YYLTYPE yylsa[YYMAXDEPTH];	/*  the location stack			*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */
  YYLTYPE *yyls = yylsa;

  int yymaxdepth = YYMAXDEPTH;

#ifndef YYPURE
  int yychar;
  YYSTYPE yylval;
  YYLTYPE yylloc;
  int yynerrs;
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
  yylsp = yyls;

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yymaxdepth - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      YYLTYPE *yyls1 = yyls;
      short *yyss1 = yyss;

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yymaxdepth);

      yyss = yyss1; yyvs = yyvs1; yyls = yyls1;
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yymaxdepth >= YYMAXLIMIT)
	yyerror("parser stack overflow");
      yymaxdepth *= 2;
      if (yymaxdepth > YYMAXLIMIT)
	yymaxdepth = YYMAXLIMIT;
      yyss = (short *) alloca (yymaxdepth * sizeof (*yyssp));
      bcopy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
      yyvs = (YYSTYPE *) alloca (yymaxdepth * sizeof (*yyvsp));
      bcopy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) alloca (yymaxdepth * sizeof (*yylsp));
      bcopy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yymaxdepth);
#endif

      if (yyssp >= yyss + yymaxdepth - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
yyresume:

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Next token is %d (%s)\n", yychar, yytname[yychar1]);
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      if (yylen == 1)
	fprintf (stderr, "Reducing 1 value via line %d, ",
		 yyrline[yyn]);
      else
	fprintf (stderr, "Reducing %d values via line %d, ",
		 yylen, yyrline[yyn]);
    }
#endif


  switch (yyn) {

case 1:
#line 219 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C forbids an empty source file");
		;
    break;}
case 2:
#line 223 "c-parse.y"
{
		  /* In case there were missing closebraces,
		     get us back to the global binding level.  */
		  while (! global_bindings_p ())
		    poplevel (0, 0, 0);
		;
    break;}
case 3:
#line 236 "c-parse.y"
{yyval.ttype = NULL_TREE; ;
    break;}
case 5:
#line 237 "c-parse.y"
{yyval.ttype = NULL_TREE; ;
    break;}
case 9:
#line 244 "c-parse.y"
{ STRIP_NOPS (yyvsp[-2].ttype);
		  if ((TREE_CODE (yyvsp[-2].ttype) == ADDR_EXPR
		       && TREE_CODE (TREE_OPERAND (yyvsp[-2].ttype, 0)) == STRING_CST)
		      || TREE_CODE (yyvsp[-2].ttype) == STRING_CST)
		    assemble_asm (yyvsp[-2].ttype);
		  else
		    error ("argument of `asm' is not a constant string"); ;
    break;}
case 10:
#line 255 "c-parse.y"
{ if (pedantic)
		    error ("ANSI C forbids data definition with no type or storage class");
		  else if (!flag_traditional)
		    warning ("data definition has no type or storage class"); ;
    break;}
case 11:
#line 260 "c-parse.y"
{;
    break;}
case 12:
#line 262 "c-parse.y"
{;
    break;}
case 13:
#line 264 "c-parse.y"
{ pedwarn ("empty declaration"); ;
    break;}
case 14:
#line 266 "c-parse.y"
{ shadow_tag (yyvsp[-1].ttype); ;
    break;}
case 17:
#line 270 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C does not allow extra `;' outside of a function"); ;
    break;}
case 18:
#line 276 "c-parse.y"
{ if (! start_function (yyvsp[-2].ttype, yyvsp[0].ttype, 0))
		    YYERROR1;
		  reinit_parse_for_function (); ;
    break;}
case 19:
#line 280 "c-parse.y"
{ store_parm_decls (); ;
    break;}
case 20:
#line 282 "c-parse.y"
{ finish_function (0); ;
    break;}
case 21:
#line 284 "c-parse.y"
{ ;
    break;}
case 22:
#line 286 "c-parse.y"
{ if (! start_function (yyvsp[-2].ttype, yyvsp[0].ttype, 0))
		    YYERROR1;
		  reinit_parse_for_function (); ;
    break;}
case 23:
#line 290 "c-parse.y"
{ store_parm_decls (); ;
    break;}
case 24:
#line 292 "c-parse.y"
{ finish_function (0); ;
    break;}
case 25:
#line 294 "c-parse.y"
{ ;
    break;}
case 26:
#line 296 "c-parse.y"
{ if (! start_function (NULL_TREE, yyvsp[0].ttype, 0))
		    YYERROR1;
		  reinit_parse_for_function (); ;
    break;}
case 27:
#line 300 "c-parse.y"
{ store_parm_decls (); ;
    break;}
case 28:
#line 302 "c-parse.y"
{ finish_function (0); ;
    break;}
case 29:
#line 304 "c-parse.y"
{ ;
    break;}
case 32:
#line 313 "c-parse.y"
{ yyval.code = ADDR_EXPR; ;
    break;}
case 33:
#line 315 "c-parse.y"
{ yyval.code = NEGATE_EXPR; ;
    break;}
case 34:
#line 317 "c-parse.y"
{ yyval.code = CONVERT_EXPR; ;
    break;}
case 35:
#line 319 "c-parse.y"
{ yyval.code = PREINCREMENT_EXPR; ;
    break;}
case 36:
#line 321 "c-parse.y"
{ yyval.code = PREDECREMENT_EXPR; ;
    break;}
case 37:
#line 323 "c-parse.y"
{ yyval.code = BIT_NOT_EXPR; ;
    break;}
case 38:
#line 325 "c-parse.y"
{ yyval.code = TRUTH_NOT_EXPR; ;
    break;}
case 39:
#line 329 "c-parse.y"
{ yyval.ttype = build_compound_expr (yyvsp[0].ttype); ;
    break;}
case 40:
#line 334 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 42:
#line 340 "c-parse.y"
{ yyval.ttype = build_tree_list (NULL_TREE, yyvsp[0].ttype); ;
    break;}
case 43:
#line 342 "c-parse.y"
{ chainon (yyvsp[-2].ttype, build_tree_list (NULL_TREE, yyvsp[0].ttype)); ;
    break;}
case 45:
#line 348 "c-parse.y"
{ yyval.ttype = build_indirect_ref (yyvsp[0].ttype, "unary *"); ;
    break;}
case 46:
#line 351 "c-parse.y"
{ yyvsp[0].itype = pedantic;
		  pedantic = 0; ;
    break;}
case 47:
#line 354 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype;
		  pedantic = yyvsp[-2].itype; ;
    break;}
case 48:
#line 357 "c-parse.y"
{ yyval.ttype = build_unary_op (yyvsp[-1].code, yyvsp[0].ttype, 0);
		  overflow_warning (yyval.ttype); ;
    break;}
case 49:
#line 361 "c-parse.y"
{ tree label = lookup_label (yyvsp[0].ttype);
		  if (label == 0)
		    yyval.ttype = null_pointer_node;
		  else
		    {
		      TREE_USED (label) = 1;
		      yyval.ttype = build1 (ADDR_EXPR, ptr_type_node, label);
		      TREE_CONSTANT (yyval.ttype) = 1;
		    }
		;
    break;}
case 50:
#line 387 "c-parse.y"
{ if (TREE_CODE (yyvsp[0].ttype) == COMPONENT_REF
		      && DECL_BIT_FIELD (TREE_OPERAND (yyvsp[0].ttype, 1)))
		    error ("`sizeof' applied to a bit-field");
		  yyval.ttype = c_sizeof (TREE_TYPE (yyvsp[0].ttype)); ;
    break;}
case 51:
#line 392 "c-parse.y"
{ yyval.ttype = c_sizeof (groktypename (yyvsp[-1].ttype)); ;
    break;}
case 52:
#line 394 "c-parse.y"
{ yyval.ttype = c_alignof_expr (yyvsp[0].ttype); ;
    break;}
case 53:
#line 396 "c-parse.y"
{ yyval.ttype = c_alignof (groktypename (yyvsp[-1].ttype)); ;
    break;}
case 54:
#line 398 "c-parse.y"
{ yyval.ttype = build_unary_op (REALPART_EXPR, yyvsp[0].ttype, 0); ;
    break;}
case 55:
#line 400 "c-parse.y"
{ yyval.ttype = build_unary_op (IMAGPART_EXPR, yyvsp[0].ttype, 0); ;
    break;}
case 57:
#line 406 "c-parse.y"
{ tree type = groktypename (yyvsp[-2].ttype);
		  yyval.ttype = build_c_cast (type, yyvsp[0].ttype); ;
    break;}
case 58:
#line 409 "c-parse.y"
{ start_init (NULL_TREE, NULL, 0);
		  yyvsp[-2].ttype = groktypename (yyvsp[-2].ttype);
		  really_start_incremental_init (yyvsp[-2].ttype); ;
    break;}
case 59:
#line 413 "c-parse.y"
{ char *name;
		  tree result = pop_init_level (0);
		  tree type = yyvsp[-5].ttype;
		  finish_init ();

		  if (pedantic)
		    pedwarn ("ANSI C forbids constructor expressions");
		  if (TYPE_NAME (type) != 0)
		    {
		      if (TREE_CODE (TYPE_NAME (type)) == IDENTIFIER_NODE)
			name = IDENTIFIER_POINTER (TYPE_NAME (type));
		      else
			name = IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (type)));
		    }
		  else
		    name = "";
		  yyval.ttype = result;
		  if (TREE_CODE (type) == ARRAY_TYPE && TYPE_SIZE (type) == 0)
		    {
		      int failure = complete_array_type (type, yyval.ttype, 1);
		      if (failure)
			abort ();
		    }
		;
    break;}
case 61:
#line 442 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 62:
#line 444 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 63:
#line 446 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 64:
#line 448 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 65:
#line 450 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 66:
#line 452 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 67:
#line 454 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 68:
#line 456 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 69:
#line 458 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 70:
#line 460 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 71:
#line 462 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 72:
#line 464 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (yyvsp[-1].code, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 73:
#line 466 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (TRUTH_ANDIF_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 74:
#line 468 "c-parse.y"
{ yyval.ttype = parser_build_binary_op (TRUTH_ORIF_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 75:
#line 470 "c-parse.y"
{ yyval.ttype = build_conditional_expr (yyvsp[-4].ttype, yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 76:
#line 472 "c-parse.y"
{ yyval.ttype = build_modify_expr (yyvsp[-2].ttype, NOP_EXPR, yyvsp[0].ttype);
		  C_SET_EXP_ORIGINAL_CODE (yyval.ttype, MODIFY_EXPR); ;
    break;}
case 77:
#line 475 "c-parse.y"
{ yyval.ttype = build_modify_expr (yyvsp[-2].ttype, yyvsp[-1].code, yyvsp[0].ttype);
		  /* This inhibits warnings in truthvalue_conversion.  */
		  C_SET_EXP_ORIGINAL_CODE (yyval.ttype, ERROR_MARK); ;
    break;}
case 78:
#line 482 "c-parse.y"
{
		  tree context;

		  yyval.ttype = lastiddecl;
		  if (!yyval.ttype || yyval.ttype == error_mark_node)
		    {
		      if (yychar == YYEMPTY)
			yychar = YYLEX;
		      if (yychar == '(')
			{
			    {
			      /* Ordinary implicit function declaration.  */
			      yyval.ttype = implicitly_declare (yyvsp[0].ttype);
			      assemble_external (yyval.ttype);
			      TREE_USED (yyval.ttype) = 1;
			    }
			}
		      else if (current_function_decl == 0)
			{
			  error ("`%s' undeclared here (not in a function)",
				 IDENTIFIER_POINTER (yyvsp[0].ttype));
			  yyval.ttype = error_mark_node;
			}
		      else
			{
			    {
			      if (IDENTIFIER_GLOBAL_VALUE (yyvsp[0].ttype) != error_mark_node
				  || IDENTIFIER_ERROR_LOCUS (yyvsp[0].ttype) != current_function_decl)
				{
				  error ("`%s' undeclared (first use this function)",
					 IDENTIFIER_POINTER (yyvsp[0].ttype));

				  if (! undeclared_variable_notice)
				    {
				      error ("(Each undeclared identifier is reported only once");
				      error ("for each function it appears in.)");
				      undeclared_variable_notice = 1;
				    }
				}
			      yyval.ttype = error_mark_node;
			      /* Prevent repeated error messages.  */
			      IDENTIFIER_GLOBAL_VALUE (yyvsp[0].ttype) = error_mark_node;
			      IDENTIFIER_ERROR_LOCUS (yyvsp[0].ttype) = current_function_decl;
			    }
			}
		    }
		  else if (TREE_TYPE (yyval.ttype) == error_mark_node)
		    yyval.ttype = error_mark_node;
		  else if (C_DECL_ANTICIPATED (yyval.ttype))
		    {
		      /* The first time we see a build-in function used,
			 if it has not been declared.  */
		      C_DECL_ANTICIPATED (yyval.ttype) = 0;
		      if (yychar == YYEMPTY)
			yychar = YYLEX;
		      if (yychar == '(')
			{
			  /* Omit the implicit declaration we
			     would ordinarily do, so we don't lose
			     the actual built in type.
			     But print a diagnostic for the mismatch.  */
			    if (TREE_CODE (yyval.ttype) != FUNCTION_DECL)
			      error ("`%s' implicitly declared as function",
				     IDENTIFIER_POINTER (DECL_NAME (yyval.ttype)));
			  else if ((TYPE_MODE (TREE_TYPE (TREE_TYPE (yyval.ttype)))
				    != TYPE_MODE (integer_type_node))
				   && (TREE_TYPE (TREE_TYPE (yyval.ttype))
				       != void_type_node))
			    pedwarn ("type mismatch in implicit declaration for built-in function `%s'",
				     IDENTIFIER_POINTER (DECL_NAME (yyval.ttype)));
			  /* If it really returns void, change that to int.  */
			  if (TREE_TYPE (TREE_TYPE (yyval.ttype)) == void_type_node)
			    TREE_TYPE (yyval.ttype)
			      = build_function_type (integer_type_node,
						     TYPE_ARG_TYPES (TREE_TYPE (yyval.ttype)));
			}
		      else
			pedwarn ("built-in function `%s' used without declaration",
				 IDENTIFIER_POINTER (DECL_NAME (yyval.ttype)));

		      /* Do what we would ordinarily do when a fn is used.  */
		      assemble_external (yyval.ttype);
		      TREE_USED (yyval.ttype) = 1;
		    }
		  else
		    {
		      assemble_external (yyval.ttype);
		      TREE_USED (yyval.ttype) = 1;
		    }

		  if (TREE_CODE (yyval.ttype) == CONST_DECL)
		    {
		      yyval.ttype = DECL_INITIAL (yyval.ttype);
		      /* This is to prevent an enum whose value is 0
			 from being considered a null pointer constant.  */
		      yyval.ttype = build1 (NOP_EXPR, TREE_TYPE (yyval.ttype), yyval.ttype);
		      TREE_CONSTANT (yyval.ttype) = 1;
		    }
		;
    break;}
case 80:
#line 583 "c-parse.y"
{ yyval.ttype = combine_strings (yyvsp[0].ttype); ;
    break;}
case 81:
#line 585 "c-parse.y"
{ char class = TREE_CODE_CLASS (TREE_CODE (yyvsp[-1].ttype));
		  if (class == 'e' || class == '1'
		      || class == '2' || class == '<')
		    C_SET_EXP_ORIGINAL_CODE (yyvsp[-1].ttype, ERROR_MARK);
		  yyval.ttype = yyvsp[-1].ttype; ;
    break;}
case 82:
#line 591 "c-parse.y"
{ yyval.ttype = error_mark_node; ;
    break;}
case 83:
#line 593 "c-parse.y"
{ if (current_function_decl == 0)
		    {
		      error ("braced-group within expression allowed only inside a function");
		      YYERROR;
		    }
		  /* We must force a BLOCK for this level
		     so that, if it is not expanded later,
		     there is a way to turn off the entire subtree of blocks
		     that are contained in it.  */
		  keep_next_level ();
		  push_iterator_stack ();
		  push_label_level ();
		  yyval.ttype = expand_start_stmt_expr (); ;
    break;}
case 84:
#line 607 "c-parse.y"
{ tree rtl_exp;
		  if (pedantic)
		    pedwarn ("ANSI C forbids braced-groups within expressions");
		  pop_iterator_stack ();
		  pop_label_level ();
		  rtl_exp = expand_end_stmt_expr (yyvsp[-2].ttype);
		  /* The statements have side effects, so the group does.  */
		  TREE_SIDE_EFFECTS (rtl_exp) = 1;

		  if (TREE_CODE (yyvsp[-1].ttype) == BLOCK)
		    {
		      /* Make a BIND_EXPR for the BLOCK already made.  */
		      yyval.ttype = build (BIND_EXPR, TREE_TYPE (rtl_exp),
				  NULL_TREE, rtl_exp, yyvsp[-1].ttype);
		      /* Remove the block from the tree at this point.
			 It gets put back at the proper place
			 when the BIND_EXPR is expanded.  */
		      delete_block (yyvsp[-1].ttype);
		    }
		  else
		    yyval.ttype = yyvsp[-1].ttype;
		;
    break;}
case 85:
#line 630 "c-parse.y"
{ yyval.ttype = build_function_call (yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 86:
#line 632 "c-parse.y"
{ yyval.ttype = build_array_ref (yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 87:
#line 634 "c-parse.y"
{
		    yyval.ttype = build_component_ref (yyvsp[-2].ttype, yyvsp[0].ttype);
		;
    break;}
case 88:
#line 638 "c-parse.y"
{
                  tree expr = build_indirect_ref (yyvsp[-2].ttype, "->");

                    yyval.ttype = build_component_ref (expr, yyvsp[0].ttype);
		;
    break;}
case 89:
#line 644 "c-parse.y"
{ yyval.ttype = build_unary_op (POSTINCREMENT_EXPR, yyvsp[-1].ttype, 0); ;
    break;}
case 90:
#line 646 "c-parse.y"
{ yyval.ttype = build_unary_op (POSTDECREMENT_EXPR, yyvsp[-1].ttype, 0); ;
    break;}
case 92:
#line 653 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 95:
#line 662 "c-parse.y"
{ c_mark_varargs ();
		  if (pedantic)
		    pedwarn ("ANSI C does not permit use of `varargs.h'"); ;
    break;}
case 96:
#line 672 "c-parse.y"
{ ;
    break;}
case 101:
#line 684 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-2].itype); ;
    break;}
case 102:
#line 688 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-2].itype); ;
    break;}
case 103:
#line 692 "c-parse.y"
{ shadow_tag_warned (yyvsp[-1].ttype, 1);
		  pedwarn ("empty declaration"); ;
    break;}
case 104:
#line 695 "c-parse.y"
{ pedwarn ("empty declaration"); ;
    break;}
case 105:
#line 704 "c-parse.y"
{ ;
    break;}
case 110:
#line 719 "c-parse.y"
{ yyval.itype = suspend_momentary ();
		  pending_xref_error ();
		  declspec_stack = tree_cons (NULL_TREE, current_declspecs,
					      declspec_stack);
		  current_declspecs = yyvsp[0].ttype; ;
    break;}
case 111:
#line 728 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-2].itype); ;
    break;}
case 112:
#line 732 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-2].itype); ;
    break;}
case 113:
#line 736 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-1].itype); ;
    break;}
case 114:
#line 740 "c-parse.y"
{ current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-1].itype); ;
    break;}
case 115:
#line 744 "c-parse.y"
{ shadow_tag (yyvsp[-1].ttype); ;
    break;}
case 116:
#line 746 "c-parse.y"
{ pedwarn ("empty declaration"); ;
    break;}
case 117:
#line 755 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 118:
#line 757 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[0].ttype, tree_cons (NULL_TREE, yyvsp[-1].ttype, yyvsp[-2].ttype)); ;
    break;}
case 119:
#line 761 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 120:
#line 763 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype); ;
    break;}
case 121:
#line 765 "c-parse.y"
{ if (extra_warnings)
		    warning ("`%s' is not at beginning of declaration",
			     IDENTIFIER_POINTER (yyvsp[0].ttype));
		  yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype); ;
    break;}
case 122:
#line 777 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, NULL_TREE);
		  TREE_STATIC (yyval.ttype) = 1; ;
    break;}
case 123:
#line 780 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 124:
#line 782 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype);
		  TREE_STATIC (yyval.ttype) = 1; ;
    break;}
case 125:
#line 785 "c-parse.y"
{ if (extra_warnings && TREE_STATIC (yyvsp[-1].ttype))
		    warning ("`%s' is not at beginning of declaration",
			     IDENTIFIER_POINTER (yyvsp[0].ttype));
		  yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype);
		  TREE_STATIC (yyval.ttype) = TREE_STATIC (yyvsp[-1].ttype); ;
    break;}
case 126:
#line 799 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 127:
#line 801 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[0].ttype, tree_cons (NULL_TREE, yyvsp[-1].ttype, yyvsp[-2].ttype)); ;
    break;}
case 128:
#line 805 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 129:
#line 807 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype); ;
    break;}
case 132:
#line 817 "c-parse.y"
{ /* For a typedef name, record the meaning, not the name.
		     In case of `foo foo, bar;'.  */
		  yyval.ttype = lookup_name (yyvsp[0].ttype); ;
    break;}
case 133:
#line 821 "c-parse.y"
{ yyval.ttype = TREE_TYPE (yyvsp[-1].ttype); ;
    break;}
case 134:
#line 823 "c-parse.y"
{ yyval.ttype = groktypename (yyvsp[-1].ttype); ;
    break;}
case 142:
#line 845 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 143:
#line 847 "c-parse.y"
{ if (TREE_CHAIN (yyvsp[-1].ttype)) yyvsp[-1].ttype = combine_strings (yyvsp[-1].ttype);
		  yyval.ttype = yyvsp[-1].ttype;
		;
    break;}
case 144:
#line 854 "c-parse.y"
{ yyval.ttype = start_decl (yyvsp[-3].ttype, current_declspecs, 1);
		  decl_attributes (yyval.ttype, yyvsp[-1].ttype);
		  start_init (yyval.ttype, yyvsp[-2].ttype, global_bindings_p ()); ;
    break;}
case 145:
#line 859 "c-parse.y"
{ finish_init ();
		  decl_attributes (yyvsp[-1].ttype, yyvsp[-3].ttype);
		  finish_decl (yyvsp[-1].ttype, yyvsp[0].ttype, yyvsp[-4].ttype); ;
    break;}
case 146:
#line 863 "c-parse.y"
{ tree d = start_decl (yyvsp[-2].ttype, current_declspecs, 0);
		  decl_attributes (d, yyvsp[0].ttype);
		  finish_decl (d, NULL_TREE, yyvsp[-1].ttype); ;
    break;}
case 147:
#line 870 "c-parse.y"
{ yyval.ttype = start_decl (yyvsp[-3].ttype, current_declspecs, 1);
		  decl_attributes (yyval.ttype, yyvsp[-1].ttype);
		  start_init (yyval.ttype, yyvsp[-2].ttype, global_bindings_p ()); ;
    break;}
case 148:
#line 875 "c-parse.y"
{ finish_init ();
		  decl_attributes (yyvsp[-1].ttype, yyvsp[-3].ttype);
		  finish_decl (yyvsp[-1].ttype, yyvsp[0].ttype, yyvsp[-4].ttype); ;
    break;}
case 149:
#line 879 "c-parse.y"
{ tree d = start_decl (yyvsp[-2].ttype, current_declspecs, 0);
		  decl_attributes (d, yyvsp[0].ttype);
		  finish_decl (d, NULL_TREE, yyvsp[-1].ttype); ;
    break;}
case 150:
#line 887 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 151:
#line 889 "c-parse.y"
{ yyval.ttype = yyvsp[-2].ttype; ;
    break;}
case 152:
#line 894 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 153:
#line 896 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-2].ttype); ;
    break;}
case 154:
#line 901 "c-parse.y"
{ if (strcmp (IDENTIFIER_POINTER (yyvsp[0].ttype), "packed")
	      && strcmp (IDENTIFIER_POINTER (yyvsp[0].ttype), "noreturn"))
	    warning ("`%s' attribute directive ignored",
		     IDENTIFIER_POINTER (yyvsp[0].ttype));
	  yyval.ttype = yyvsp[0].ttype; ;
    break;}
case 156:
#line 908 "c-parse.y"
{ /* If not "mode (m)", then issue warning.  */
	  if (strcmp (IDENTIFIER_POINTER (yyvsp[-3].ttype), "mode") != 0)
	    {
	      warning ("`%s' attribute directive ignored",
		       IDENTIFIER_POINTER (yyvsp[-3].ttype));
	      yyval.ttype = yyvsp[-3].ttype;
	    }
	  else
	    yyval.ttype = tree_cons (yyvsp[-3].ttype, yyvsp[-1].ttype, NULL_TREE); ;
    break;}
case 157:
#line 918 "c-parse.y"
{ /* if not "aligned(n)", then issue warning */
	  if (strcmp (IDENTIFIER_POINTER (yyvsp[-3].ttype), "aligned") != 0
	      || TREE_CODE (yyvsp[-1].ttype) != INTEGER_CST)
	    {
	      warning ("`%s' attribute directive ignored",
		       IDENTIFIER_POINTER (yyvsp[-3].ttype));
	      yyval.ttype = yyvsp[-3].ttype;
	    }
	  else
	    yyval.ttype = tree_cons (yyvsp[-3].ttype, yyvsp[-1].ttype, NULL_TREE); ;
    break;}
case 158:
#line 929 "c-parse.y"
{ /* if not "format(...)", then issue warning */
	  if (strcmp (IDENTIFIER_POINTER (yyvsp[-7].ttype), "format") != 0
	      || TREE_CODE (yyvsp[-3].ttype) != INTEGER_CST
	      || TREE_CODE (yyvsp[-1].ttype) != INTEGER_CST)
	    {
	      warning ("`%s' attribute directive ignored",
		       IDENTIFIER_POINTER (yyvsp[-7].ttype));
	      yyval.ttype = yyvsp[-7].ttype;
	    }
	  else
	    yyval.ttype = tree_cons (yyvsp[-7].ttype,
			    tree_cons (yyvsp[-5].ttype,
				       tree_cons (yyvsp[-3].ttype, yyvsp[-1].ttype, NULL_TREE),
				       NULL_TREE),
			    NULL_TREE); ;
    break;}
case 160:
#line 951 "c-parse.y"
{ really_start_incremental_init (NULL_TREE);
		  /* Note that the call to clear_momentary
		     is in process_init_element.  */
		  push_momentary (); ;
    break;}
case 161:
#line 956 "c-parse.y"
{ yyval.ttype = pop_init_level (0);
		  if (yyval.ttype == error_mark_node)
		    pop_momentary ();
		  else
		    pop_momentary_nofree (); ;
    break;}
case 162:
#line 963 "c-parse.y"
{ yyval.ttype = error_mark_node; ;
    break;}
case 163:
#line 969 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C forbids empty initializer braces"); ;
    break;}
case 167:
#line 983 "c-parse.y"
{ process_init_element (yyvsp[0].ttype); ;
    break;}
case 168:
#line 985 "c-parse.y"
{ push_init_level (0); ;
    break;}
case 169:
#line 987 "c-parse.y"
{ process_init_element (pop_init_level (0)); ;
    break;}
case 171:
#line 993 "c-parse.y"
{ set_init_index (yyvsp[-4].ttype, yyvsp[-2].ttype); ;
    break;}
case 173:
#line 996 "c-parse.y"
{ set_init_index (yyvsp[-2].ttype, NULL_TREE); ;
    break;}
case 175:
#line 999 "c-parse.y"
{ set_init_label (yyvsp[-1].ttype); ;
    break;}
case 177:
#line 1002 "c-parse.y"
{ set_init_label (yyvsp[-1].ttype); ;
    break;}
case 179:
#line 1008 "c-parse.y"
{ push_c_function_context ();
		  if (! start_function (current_declspecs, yyvsp[0].ttype, 1))
		    {
		      pop_c_function_context ();
		      YYERROR1;
		    }
		  reinit_parse_for_function ();
		  store_parm_decls (); ;
    break;}
case 180:
#line 1023 "c-parse.y"
{ finish_function (1);
		  pop_c_function_context (); ;
    break;}
case 181:
#line 1029 "c-parse.y"
{ push_c_function_context ();
		  if (! start_function (current_declspecs, yyvsp[0].ttype, 1))
		    {
		      pop_c_function_context ();
		      YYERROR1;
		    }
		  reinit_parse_for_function ();
		  store_parm_decls (); ;
    break;}
case 182:
#line 1044 "c-parse.y"
{ finish_function (1);
		  pop_c_function_context (); ;
    break;}
case 185:
#line 1060 "c-parse.y"
{ yyval.ttype = yyvsp[-1].ttype; ;
    break;}
case 186:
#line 1062 "c-parse.y"
{ yyval.ttype = build_nt (CALL_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 187:
#line 1067 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 188:
#line 1069 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-2].ttype, NULL_TREE); ;
    break;}
case 189:
#line 1071 "c-parse.y"
{ yyval.ttype = make_pointer_declarator (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 191:
#line 1082 "c-parse.y"
{ yyval.ttype = build_nt (CALL_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 192:
#line 1087 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 193:
#line 1089 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-2].ttype, NULL_TREE); ;
    break;}
case 194:
#line 1091 "c-parse.y"
{ yyval.ttype = make_pointer_declarator (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 196:
#line 1100 "c-parse.y"
{ yyval.ttype = build_nt (CALL_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 197:
#line 1105 "c-parse.y"
{ yyval.ttype = yyvsp[-1].ttype; ;
    break;}
case 198:
#line 1107 "c-parse.y"
{ yyval.ttype = make_pointer_declarator (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 199:
#line 1109 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 200:
#line 1111 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-2].ttype, NULL_TREE); ;
    break;}
case 202:
#line 1117 "c-parse.y"
{ yyval.ttype = start_struct (RECORD_TYPE, yyvsp[-1].ttype);
		  /* Start scope of tag before parsing components.  */
		;
    break;}
case 203:
#line 1121 "c-parse.y"
{ yyval.ttype = finish_struct (yyvsp[-2].ttype, yyvsp[-1].ttype);
		  /* Really define the structure.  */
		;
    break;}
case 204:
#line 1125 "c-parse.y"
{ yyval.ttype = finish_struct (start_struct (RECORD_TYPE, NULL_TREE),
				      yyvsp[-1].ttype); ;
    break;}
case 205:
#line 1128 "c-parse.y"
{ yyval.ttype = xref_tag (RECORD_TYPE, yyvsp[0].ttype); ;
    break;}
case 206:
#line 1130 "c-parse.y"
{ yyval.ttype = start_struct (UNION_TYPE, yyvsp[-1].ttype); ;
    break;}
case 207:
#line 1132 "c-parse.y"
{ yyval.ttype = finish_struct (yyvsp[-2].ttype, yyvsp[-1].ttype); ;
    break;}
case 208:
#line 1134 "c-parse.y"
{ yyval.ttype = finish_struct (start_struct (UNION_TYPE, NULL_TREE),
				      yyvsp[-1].ttype); ;
    break;}
case 209:
#line 1137 "c-parse.y"
{ yyval.ttype = xref_tag (UNION_TYPE, yyvsp[0].ttype); ;
    break;}
case 210:
#line 1139 "c-parse.y"
{ yyvsp[0].itype = suspend_momentary ();
		  yyval.ttype = start_enum (yyvsp[-1].ttype); ;
    break;}
case 211:
#line 1142 "c-parse.y"
{ yyval.ttype = finish_enum (yyvsp[-3].ttype, nreverse (yyvsp[-2].ttype));
		  resume_momentary (yyvsp[-4].itype); ;
    break;}
case 212:
#line 1145 "c-parse.y"
{ yyvsp[0].itype = suspend_momentary ();
		  yyval.ttype = start_enum (NULL_TREE); ;
    break;}
case 213:
#line 1148 "c-parse.y"
{ yyval.ttype = finish_enum (yyvsp[-3].ttype, nreverse (yyvsp[-2].ttype));
		  resume_momentary (yyvsp[-4].itype); ;
    break;}
case 214:
#line 1151 "c-parse.y"
{ yyval.ttype = xref_tag (ENUMERAL_TYPE, yyvsp[0].ttype); ;
    break;}
case 218:
#line 1162 "c-parse.y"
{ if (pedantic) pedwarn ("comma at end of enumerator list"); ;
    break;}
case 219:
#line 1167 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype; ;
    break;}
case 220:
#line 1169 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-1].ttype, yyvsp[0].ttype);
		  pedwarn ("no semicolon at end of struct or union"); ;
    break;}
case 221:
#line 1174 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 222:
#line 1176 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-2].ttype, yyvsp[-1].ttype); ;
    break;}
case 223:
#line 1178 "c-parse.y"
{ if (pedantic)
		    pedwarn ("extra semicolon in struct or union specified"); ;
    break;}
case 224:
#line 1193 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype;
		  current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-1].itype); ;
    break;}
case 225:
#line 1198 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C forbids member declarations with no members");
		  shadow_tag(yyvsp[0].ttype);
		  yyval.ttype = NULL_TREE; ;
    break;}
case 226:
#line 1203 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype;
		  current_declspecs = TREE_VALUE (declspec_stack);
		  declspec_stack = TREE_CHAIN (declspec_stack);
		  resume_momentary (yyvsp[-1].itype); ;
    break;}
case 227:
#line 1208 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C forbids member declarations with no members");
		  shadow_tag(yyvsp[0].ttype);
		  yyval.ttype = NULL_TREE; ;
    break;}
case 228:
#line 1213 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 230:
#line 1219 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 231:
#line 1224 "c-parse.y"
{ yyval.ttype = grokfield (yyvsp[-3].filename, yyvsp[-2].lineno, yyvsp[-1].ttype, current_declspecs, NULL_TREE);
		  decl_attributes (yyval.ttype, yyvsp[0].ttype); ;
    break;}
case 232:
#line 1228 "c-parse.y"
{ yyval.ttype = grokfield (yyvsp[-5].filename, yyvsp[-4].lineno, yyvsp[-3].ttype, current_declspecs, yyvsp[-1].ttype);
		  decl_attributes (yyval.ttype, yyvsp[0].ttype); ;
    break;}
case 233:
#line 1231 "c-parse.y"
{ yyval.ttype = grokfield (yyvsp[-4].filename, yyvsp[-3].lineno, NULL_TREE, current_declspecs, yyvsp[-1].ttype);
		  decl_attributes (yyval.ttype, yyvsp[0].ttype); ;
    break;}
case 235:
#line 1243 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[0].ttype, yyvsp[-2].ttype); ;
    break;}
case 236:
#line 1249 "c-parse.y"
{ yyval.ttype = build_enumerator (yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 237:
#line 1251 "c-parse.y"
{ yyval.ttype = build_enumerator (yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 238:
#line 1256 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 239:
#line 1258 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 240:
#line 1263 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 242:
#line 1269 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 243:
#line 1271 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype); ;
    break;}
case 244:
#line 1276 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 245:
#line 1278 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, yyvsp[0].ttype, yyvsp[-1].ttype); ;
    break;}
case 246:
#line 1283 "c-parse.y"
{ yyval.ttype = yyvsp[-1].ttype; ;
    break;}
case 247:
#line 1286 "c-parse.y"
{ yyval.ttype = make_pointer_declarator (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 248:
#line 1288 "c-parse.y"
{ yyval.ttype = make_pointer_declarator (yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 249:
#line 1290 "c-parse.y"
{ yyval.ttype = build_nt (CALL_EXPR, yyvsp[-2].ttype, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 250:
#line 1292 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 251:
#line 1294 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, yyvsp[-2].ttype, NULL_TREE); ;
    break;}
case 252:
#line 1296 "c-parse.y"
{ yyval.ttype = build_nt (CALL_EXPR, NULL_TREE, yyvsp[0].ttype, NULL_TREE); ;
    break;}
case 253:
#line 1298 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, NULL_TREE, yyvsp[-1].ttype); ;
    break;}
case 254:
#line 1300 "c-parse.y"
{ yyval.ttype = build_nt (ARRAY_REF, NULL_TREE, NULL_TREE); ;
    break;}
case 261:
#line 1322 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  pushlevel (0);
		  clear_last_expr ();
		  push_momentary ();
		  expand_start_bindings (0);
		;
    break;}
case 263:
#line 1335 "c-parse.y"
{ if (pedantic)
		    pedwarn ("ANSI C forbids label declarations"); ;
    break;}
case 266:
#line 1346 "c-parse.y"
{ tree link;
		  for (link = yyvsp[-1].ttype; link; link = TREE_CHAIN (link))
		    {
		      tree label = shadow_label (TREE_VALUE (link));
		      C_DECLARED_LABEL_FLAG (label) = 1;
		      declare_nonlocal_label (label);
		    }
		;
    break;}
case 267:
#line 1360 "c-parse.y"
{;
    break;}
case 269:
#line 1365 "c-parse.y"
{ yyval.ttype = convert (void_type_node, integer_zero_node); ;
    break;}
case 270:
#line 1367 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  expand_end_bindings (getdecls (), 1, 0);
		  yyval.ttype = poplevel (1, 1, 0);
		  pop_momentary (); ;
    break;}
case 271:
#line 1372 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  expand_end_bindings (getdecls (), kept_level_p (), 0);
		  yyval.ttype = poplevel (kept_level_p (), 0, 0);
		  pop_momentary (); ;
    break;}
case 272:
#line 1377 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  expand_end_bindings (getdecls (), kept_level_p (), 0);
		  yyval.ttype = poplevel (kept_level_p (), 0, 0);
		  pop_momentary (); ;
    break;}
case 275:
#line 1394 "c-parse.y"
{ emit_line_note (yyvsp[-5].filename, yyvsp[-4].lineno);
		  expand_start_cond (truthvalue_conversion (yyvsp[-1].ttype), 0);
		  yyval.itype = stmt_count;
		  if_stmt_file = yyvsp[-5].filename;
		  if_stmt_line = yyvsp[-4].lineno;
		  position_after_white_space (); ;
    break;}
case 276:
#line 1407 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-2].filename, yyvsp[-1].lineno);
		  /* See comment in `while' alternative, above.  */
		  emit_nop ();
		  expand_start_loop_continue_elsewhere (1);
		  position_after_white_space (); ;
    break;}
case 277:
#line 1414 "c-parse.y"
{ expand_loop_continue_here (); ;
    break;}
case 278:
#line 1418 "c-parse.y"
{ yyval.filename = input_filename; ;
    break;}
case 279:
#line 1422 "c-parse.y"
{ yyval.lineno = lineno; ;
    break;}
case 280:
#line 1427 "c-parse.y"
{ ;
    break;}
case 281:
#line 1432 "c-parse.y"
{ ;
    break;}
case 282:
#line 1437 "c-parse.y"
{ ;
    break;}
case 284:
#line 1443 "c-parse.y"
{ int next;
		  position_after_white_space ();
		  next = getc (finput);
		  ungetc (next, finput);
		  if (pedantic && next == '}')
		    pedwarn ("ANSI C forbids label at end of compound statement");
		;
    break;}
case 285:
#line 1455 "c-parse.y"
{ stmt_count++; ;
    break;}
case 287:
#line 1458 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-3].filename, yyvsp[-2].lineno);
/* It appears that this should not be done--that a non-lvalue array
   shouldn't get an error if the value isn't used.
   Section 3.2.2.1 says that an array lvalue gets converted to a pointer
   if it appears as a top-level expression,
   but says nothing about non-lvalue arrays.  */
#if 0
		  /* Call default_conversion to get an error
		     on referring to a register array if pedantic.  */
		  if (TREE_CODE (TREE_TYPE (yyvsp[-1].ttype)) == ARRAY_TYPE
		      || TREE_CODE (TREE_TYPE (yyvsp[-1].ttype)) == FUNCTION_TYPE)
		    yyvsp[-1].ttype = default_conversion (yyvsp[-1].ttype);
#endif
		  iterator_expand (yyvsp[-1].ttype);
		  clear_momentary (); ;
    break;}
case 288:
#line 1475 "c-parse.y"
{ expand_start_else ();
		  yyvsp[-1].itype = stmt_count;
		  position_after_white_space (); ;
    break;}
case 289:
#line 1479 "c-parse.y"
{ expand_end_cond ();
		  if (extra_warnings && stmt_count == yyvsp[-3].itype)
		    warning ("empty body in an else-statement"); ;
    break;}
case 290:
#line 1483 "c-parse.y"
{ expand_end_cond ();
		  /* This warning is here instead of in simple_if, because we
		     do not want a warning if an empty if is followed by an
		     else statement.  */
		  if (extra_warnings && stmt_count == yyvsp[0].itype)
		    warning_with_file_and_line (if_stmt_file, if_stmt_line,
						"empty body in an if-statement"); ;
    break;}
case 291:
#line 1494 "c-parse.y"
{ expand_end_cond (); ;
    break;}
case 292:
#line 1496 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-2].filename, yyvsp[-1].lineno);
		  /* The emit_nop used to come before emit_line_note,
		     but that made the nop seem like part of the preceding line.
		     And that was confusing when the preceding line was
		     inside of an if statement and was not really executed.
		     I think it ought to work to put the nop after the line number.
		     We will see.  --rms, July 15, 1991.  */
		  emit_nop (); ;
    break;}
case 293:
#line 1506 "c-parse.y"
{ /* Don't start the loop till we have succeeded
		     in parsing the end test.  This is to make sure
		     that we end every loop we start.  */
		  expand_start_loop (1);
		  emit_line_note (input_filename, lineno);
		  expand_exit_loop_if_false (NULL_PTR,
					     truthvalue_conversion (yyvsp[-1].ttype));
		  position_after_white_space (); ;
    break;}
case 294:
#line 1515 "c-parse.y"
{ expand_end_loop (); ;
    break;}
case 295:
#line 1518 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  expand_exit_loop_if_false (NULL_PTR,
					     truthvalue_conversion (yyvsp[-2].ttype));
		  expand_end_loop ();
		  clear_momentary (); ;
    break;}
case 296:
#line 1525 "c-parse.y"
{ expand_end_loop ();
		  clear_momentary (); ;
    break;}
case 297:
#line 1529 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-5].filename, yyvsp[-4].lineno);
		  /* See comment in `while' alternative, above.  */
		  emit_nop ();
		  if (yyvsp[-1].ttype) c_expand_expr_stmt (yyvsp[-1].ttype);
		  /* Next step is to call expand_start_loop_continue_elsewhere,
		     but wait till after we parse the entire for (...).
		     Otherwise, invalid input might cause us to call that
		     fn without calling expand_end_loop.  */
		;
    break;}
case 298:
#line 1541 "c-parse.y"
{ yyvsp[0].lineno = lineno;
		  yyval.filename = input_filename; ;
    break;}
case 299:
#line 1544 "c-parse.y"
{ 
		  /* Start the loop.  Doing this after parsing
		     all the expressions ensures we will end the loop.  */
		  expand_start_loop_continue_elsewhere (1);
		  /* Emit the end-test, with a line number.  */
		  emit_line_note (yyvsp[-2].filename, yyvsp[-3].lineno);
		  if (yyvsp[-4].ttype)
		    expand_exit_loop_if_false (NULL_PTR,
					       truthvalue_conversion (yyvsp[-4].ttype));
		  /* Don't let the tree nodes for $9 be discarded by
		     clear_momentary during the parsing of the next stmt.  */
		  push_momentary ();
		  yyvsp[-3].lineno = lineno;
		  yyvsp[-2].filename = input_filename;
		  position_after_white_space (); ;
    break;}
case 300:
#line 1560 "c-parse.y"
{ /* Emit the increment expression, with a line number.  */
		  emit_line_note (yyvsp[-4].filename, yyvsp[-5].lineno);
		  expand_loop_continue_here ();
		  if (yyvsp[-3].ttype)
		    c_expand_expr_stmt (yyvsp[-3].ttype);
		  pop_momentary ();
		  expand_end_loop (); ;
    break;}
case 301:
#line 1568 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-5].filename, yyvsp[-4].lineno);
		  c_expand_start_case (yyvsp[-1].ttype);
		  /* Don't let the tree nodes for $3 be discarded by
		     clear_momentary during the parsing of the next stmt.  */
		  push_momentary ();
		  position_after_white_space (); ;
    break;}
case 302:
#line 1576 "c-parse.y"
{ expand_end_case (yyvsp[-3].ttype);
		  pop_momentary (); ;
    break;}
case 303:
#line 1579 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-3].filename, yyvsp[-2].lineno);
		  if ( ! expand_exit_something ())
		    error ("break statement not within loop or switch"); ;
    break;}
case 304:
#line 1584 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-3].filename, yyvsp[-2].lineno);
		  if (! expand_continue_loop (NULL_PTR))
		    error ("continue statement not within a loop"); ;
    break;}
case 305:
#line 1589 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-3].filename, yyvsp[-2].lineno);
		  c_expand_return (NULL_TREE); ;
    break;}
case 306:
#line 1593 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-4].filename, yyvsp[-3].lineno);
		  c_expand_return (yyvsp[-1].ttype); ;
    break;}
case 307:
#line 1597 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-7].filename, yyvsp[-6].lineno);
		  STRIP_NOPS (yyvsp[-2].ttype);
		  if ((TREE_CODE (yyvsp[-2].ttype) == ADDR_EXPR
		       && TREE_CODE (TREE_OPERAND (yyvsp[-2].ttype, 0)) == STRING_CST)
		      || TREE_CODE (yyvsp[-2].ttype) == STRING_CST)
		    expand_asm (yyvsp[-2].ttype);
		  else
		    error ("argument of `asm' is not a constant string"); ;
    break;}
case 308:
#line 1608 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-9].filename, yyvsp[-8].lineno);
		  c_expand_asm_operands (yyvsp[-4].ttype, yyvsp[-2].ttype, NULL_TREE, NULL_TREE,
					 yyvsp[-6].ttype == ridpointers[(int)RID_VOLATILE],
					 input_filename, lineno); ;
    break;}
case 309:
#line 1615 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-11].filename, yyvsp[-10].lineno);
		  c_expand_asm_operands (yyvsp[-6].ttype, yyvsp[-4].ttype, yyvsp[-2].ttype, NULL_TREE,
					 yyvsp[-8].ttype == ridpointers[(int)RID_VOLATILE],
					 input_filename, lineno); ;
    break;}
case 310:
#line 1623 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-13].filename, yyvsp[-12].lineno);
		  c_expand_asm_operands (yyvsp[-8].ttype, yyvsp[-6].ttype, yyvsp[-4].ttype, yyvsp[-2].ttype,
					 yyvsp[-10].ttype == ridpointers[(int)RID_VOLATILE],
					 input_filename, lineno); ;
    break;}
case 311:
#line 1629 "c-parse.y"
{ tree decl;
		  stmt_count++;
		  emit_line_note (yyvsp[-4].filename, yyvsp[-3].lineno);
		  decl = lookup_label (yyvsp[-1].ttype);
		  if (decl != 0)
		    {
		      TREE_USED (decl) = 1;
		      expand_goto (decl);
		    }
		;
    break;}
case 312:
#line 1640 "c-parse.y"
{ stmt_count++;
		  emit_line_note (yyvsp[-5].filename, yyvsp[-4].lineno);
		  expand_computed_goto (convert (ptr_type_node, yyvsp[-1].ttype)); ;
    break;}
case 315:
#line 1653 "c-parse.y"
{
	    /* The value returned by this action is  */
	    /*      1 if everything is OK */ 
	    /*      0 in case of error or already bound iterator */

	    yyval.itype = 0;
	    if (TREE_CODE (yyvsp[-1].ttype) != VAR_DECL)
	      error ("invalid `for (ITERATOR)' syntax");
	    else if (! ITERATOR_P (yyvsp[-1].ttype))
	      error ("`%s' is not an iterator",
		     IDENTIFIER_POINTER (DECL_NAME (yyvsp[-1].ttype)));
	    else if (ITERATOR_BOUND_P (yyvsp[-1].ttype))
	      error ("`for (%s)' inside expansion of same iterator",
		     IDENTIFIER_POINTER (DECL_NAME (yyvsp[-1].ttype)));
	    else
	      {
		yyval.itype = 1;
		iterator_for_loop_start (yyvsp[-1].ttype);
	      }
	  ;
    break;}
case 316:
#line 1674 "c-parse.y"
{
	    if (yyvsp[-1].itype)
	      iterator_for_loop_end (yyvsp[-3].ttype);
	  ;
    break;}
case 317:
#line 1706 "c-parse.y"
{ register tree value = check_case_value (yyvsp[-1].ttype);
		  register tree label
		    = build_decl (LABEL_DECL, NULL_TREE, NULL_TREE);

		  stmt_count++;

		  if (value != error_mark_node)
		    {
		      tree duplicate;
		      int success = pushcase (value, convert_and_check,
					      label, &duplicate);
		      if (success == 1)
			error ("case label not within a switch statement");
		      else if (success == 2)
			{
			  error ("duplicate case value");
			  error_with_decl (duplicate, "this is the first entry for that value");
			}
		      else if (success == 3)
			warning ("case value out of range");
		      else if (success == 5)
			error ("case label within scope of cleanup or variable array");
		    }
		  position_after_white_space (); ;
    break;}
case 318:
#line 1731 "c-parse.y"
{ register tree value1 = check_case_value (yyvsp[-3].ttype);
		  register tree value2 = check_case_value (yyvsp[-1].ttype);
		  register tree label
		    = build_decl (LABEL_DECL, NULL_TREE, NULL_TREE);

		  stmt_count++;

		  if (value1 != error_mark_node && value2 != error_mark_node)
		    {
		      tree duplicate;
		      int success = pushcase_range (value1, value2,
						    convert_and_check, label,
						    &duplicate);
		      if (success == 1)
			error ("case label not within a switch statement");
		      else if (success == 2)
			{
			  error ("duplicate case value");
			  error_with_decl (duplicate, "this is the first entry for that value");
			}
		      else if (success == 3)
			warning ("case value out of range");
		      else if (success == 4)
			warning ("empty case range");
		      else if (success == 5)
			error ("case label within scope of cleanup or variable array");
		    }
		  position_after_white_space (); ;
    break;}
case 319:
#line 1760 "c-parse.y"
{
		  tree duplicate;
		  register tree label
		    = build_decl (LABEL_DECL, NULL_TREE, NULL_TREE);
		  int success = pushcase (NULL_TREE, 0, label, &duplicate);
		  stmt_count++;
		  if (success == 1)
		    error ("default label not within a switch statement");
		  else if (success == 2)
		    {
		      error ("multiple default labels in one switch");
		      error_with_decl (duplicate, "this is the first default label");
		    }
		  position_after_white_space (); ;
    break;}
case 320:
#line 1775 "c-parse.y"
{ tree label = define_label (input_filename, lineno, yyvsp[-1].ttype);
		  stmt_count++;
		  emit_nop ();
		  if (label)
		    expand_label (label);
		  position_after_white_space (); ;
    break;}
case 321:
#line 1787 "c-parse.y"
{ emit_line_note (input_filename, lineno);
		  yyval.ttype = NULL_TREE; ;
    break;}
case 322:
#line 1790 "c-parse.y"
{ emit_line_note (input_filename, lineno); ;
    break;}
case 323:
#line 1795 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 325:
#line 1802 "c-parse.y"
{ yyval.ttype = NULL_TREE; ;
    break;}
case 328:
#line 1809 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-2].ttype, yyvsp[0].ttype); ;
    break;}
case 329:
#line 1814 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-3].ttype, yyvsp[-1].ttype); ;
    break;}
case 330:
#line 1819 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, combine_strings (yyvsp[0].ttype), NULL_TREE); ;
    break;}
case 331:
#line 1821 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, combine_strings (yyvsp[0].ttype), yyvsp[-2].ttype); ;
    break;}
case 332:
#line 1827 "c-parse.y"
{ pushlevel (0);
		  clear_parm_order ();
		  declare_parm_level (0); ;
    break;}
case 333:
#line 1831 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype;
		  parmlist_tags_warning ();
		  poplevel (0, 0, 0); ;
    break;}
case 335:
#line 1839 "c-parse.y"
{ tree parm;
		  if (pedantic)
		    pedwarn ("ANSI C forbids forward parameter declarations");
		  /* Mark the forward decls as such.  */
		  for (parm = getdecls (); parm; parm = TREE_CHAIN (parm))
		    TREE_ASM_WRITTEN (parm) = 1;
		  clear_parm_order (); ;
    break;}
case 336:
#line 1847 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype; ;
    break;}
case 337:
#line 1849 "c-parse.y"
{ yyval.ttype = tree_cons (NULL_TREE, NULL_TREE, NULL_TREE); ;
    break;}
case 338:
#line 1855 "c-parse.y"
{ yyval.ttype = get_parm_info (0); ;
    break;}
case 339:
#line 1857 "c-parse.y"
{ yyval.ttype = get_parm_info (0);
		  if (pedantic)
		    pedwarn ("ANSI C requires a named argument before `...'");
		;
    break;}
case 340:
#line 1862 "c-parse.y"
{ yyval.ttype = get_parm_info (1); ;
    break;}
case 341:
#line 1864 "c-parse.y"
{ yyval.ttype = get_parm_info (0); ;
    break;}
case 342:
#line 1869 "c-parse.y"
{ push_parm_decl (yyvsp[0].ttype); ;
    break;}
case 343:
#line 1871 "c-parse.y"
{ push_parm_decl (yyvsp[0].ttype); ;
    break;}
case 344:
#line 1878 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype)	; ;
    break;}
case 345:
#line 1880 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype)	; ;
    break;}
case 346:
#line 1882 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 347:
#line 1884 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype)	; ;
    break;}
case 348:
#line 1886 "c-parse.y"
{ yyval.ttype = build_tree_list (yyvsp[-1].ttype, yyvsp[0].ttype); ;
    break;}
case 349:
#line 1893 "c-parse.y"
{ pushlevel (0);
		  clear_parm_order ();
		  declare_parm_level (1); ;
    break;}
case 350:
#line 1897 "c-parse.y"
{ yyval.ttype = yyvsp[0].ttype;
		  parmlist_tags_warning ();
		  poplevel (0, 0, 0); ;
    break;}
case 352:
#line 1905 "c-parse.y"
{ tree t;
		  for (t = yyvsp[-1].ttype; t; t = TREE_CHAIN (t))
		    if (TREE_VALUE (t) == NULL_TREE)
		      error ("`...' in old-style identifier list");
		  yyval.ttype = tree_cons (NULL_TREE, NULL_TREE, yyvsp[-1].ttype); ;
    break;}
case 353:
#line 1915 "c-parse.y"
{ yyval.ttype = build_tree_list (NULL_TREE, yyvsp[0].ttype); ;
    break;}
case 354:
#line 1917 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-2].ttype, build_tree_list (NULL_TREE, yyvsp[0].ttype)); ;
    break;}
case 355:
#line 1923 "c-parse.y"
{ yyval.ttype = build_tree_list (NULL_TREE, yyvsp[0].ttype); ;
    break;}
case 356:
#line 1925 "c-parse.y"
{ yyval.ttype = chainon (yyvsp[-2].ttype, build_tree_list (NULL_TREE, yyvsp[0].ttype)); ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 327 "bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;
      yyerror("parse error");
    }

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;
}
#line 1928 "c-parse.y"

