typedef union {long itype; tree ttype; char *strtype; enum tree_code code; } YYSTYPE;
#define	IDENTIFIER	258
#define	TYPENAME	259
#define	SCOPED_TYPENAME	260
#define	SCSPEC	261
#define	TYPESPEC	262
#define	TYPE_QUAL	263
#define	CONSTANT	264
#define	STRING	265
#define	ELLIPSIS	266
#define	SIZEOF	267
#define	ENUM	268
#define	IF	269
#define	ELSE	270
#define	WHILE	271
#define	DO	272
#define	FOR	273
#define	SWITCH	274
#define	CASE	275
#define	DEFAULT	276
#define	BREAK	277
#define	CONTINUE	278
#define	RETURN	279
#define	GOTO	280
#define	ASM_KEYWORD	281
#define	GCC_ASM_KEYWORD	282
#define	TYPEOF	283
#define	ALIGNOF	284
#define	HEADOF	285
#define	CLASSOF	286
#define	ATTRIBUTE	287
#define	EXTENSION	288
#define	LABEL	289
#define	AGGR	290
#define	VISSPEC	291
#define	DELETE	292
#define	NEW	293
#define	OVERLOAD	294
#define	THIS	295
#define	OPERATOR	296
#define	LEFT_RIGHT	297
#define	TEMPLATE	298
#define	TYPEID	299
#define	DYNAMIC_CAST	300
#define	SCOPE	301
#define	START_DECLARATOR	302
#define	EMPTY	303
#define	TYPENAME_COLON	304
#define	ASSIGN	305
#define	RANGE	306
#define	OROR	307
#define	ANDAND	308
#define	MIN_MAX	309
#define	EQCOMPARE	310
#define	ARITHCOMPARE	311
#define	LSHIFT	312
#define	RSHIFT	313
#define	UNARY	314
#define	PLUSPLUS	315
#define	MINUSMINUS	316
#define	HYPERUNARY	317
#define	PAREN_STAR_PAREN	318
#define	POINTSAT	319
#define	POINTSAT_STAR	320
#define	DOT_STAR	321
#define	RAISE	322
#define	RAISES	323
#define	RERAISE	324
#define	TRY	325
#define	EXCEPT	326
#define	CATCH	327
#define	THROW	328
#define	ANSI_TRY	329
#define	ANSI_THROW	330
#define	TYPENAME_ELLIPSIS	331
#define	PTYPENAME	332
#define	PRE_PARSED_FUNCTION_DECL	333
#define	EXTERN_LANG_STRING	334
#define	ALL	335
#define	PRE_PARSED_CLASS_DECL	336
#define	TYPENAME_DEFN	337
#define	IDENTIFIER_DEFN	338
#define	PTYPENAME_DEFN	339
#define	END_OF_SAVED_INPUT	340


extern YYSTYPE yylval;
#define YYEMPTY		-2
